//TCNAMS1B JOB (ACCT#),BR136,
// NOTIFY=TCNAMS1,
// CLASS=A,MSGCLASS=X,COND=(0,NE)
//ASMH EXEC PGM=IEV90,PARM=(NODECK,OBJECT,NOESD,NORLD,NOXREF)
***********************************************************************
*                                                                     *
*        "BR" Command                                                 *
*                                                                     *
*        This ISPF/PDF program allows you to browse most VSAM,        *
*        BDAM, and multi-volume data sets.                            *
*                                                                     *
*        BR may be invoked as a TSO command, as a program,            *
*        or as an edit macro.                                         *
*                                                                     *
*        1. TSO command                                               *
*                                                                     *
*          Syntax:                                                    *
*                                                                     *
*            BR dsname VOL(volser)                                    *
*                                                                     *
*        2. EDIT macro                                                *
*                                                                     *
*          Syntax:                                                    *
*                                                                     *
*            !BR dsname                                               *
*                                                                     *
*        3. Program                                                   *
*                                                                     *
*            BR may be invoked via the ISPF SELECT service, the       *
*            TSO CALL command, or a LINK macro.  A fully-qualified    *
*            dsname may be specified in the parm.                     *
*                                                                     *
*        BR first determines if the specified data set is             *
*        NOT supported by ISPF/PDF's BROWSE service, i.e. one         *
*        of the following:                                            *
*                                                                     *
*          1.  VSAM (ESDS, KSDS, RRDS, LDS, VVDS)                     *
*          2.  BDAM                                                   *
*          3.  multi-volume                                           *
*          4.  BLKSIZE=0 or DSORG=0 or KEYLEN>0 or DS1LSTAR=0         *
*              or RECFM=VBS                                           *
*          5.  data set allocated to another job with DISP=OLD        *
*                                                                     *
*        Based on previous determination, BR will:                    *
*                                                                     *
*          a. Invoke the BROWSE service to process the data set,      *
*             if BROWSE supports it.                                  *
*                                                                     *
*          b. allocate the data set, process it using VSAM or EXCP,   *
*             and display the data via BRIF (the BROWSE interface),   *
*             if BROWSE does not support it.                          *
*                                                                     *
*    NOTES:                                                           *
*                                                                     *
*      a.  The dsname is optional.  When no dsname is specified,      *
*          BR tries to find a dsname on the screen (at the cursor     *
*          position).  If this fails, then BR prompts the user to     *
*          enter the dsname.                                          *
*                                                                     *
*      b.  If BR is invoked as a command or an edit macro             *
*          with a dsname that is not enclosed in quotes,              *
*          BR prefixes it with the userid.                            *
*                                                                     *
*      c.  A volume serial is optional.  BR assumes that the          *
*          data set is cataloged if no volser is specified.           *
*          If the data set is not cataloged, BR uses the ZDLVOL       *
*          variable (set when a command is invoked on the Data Set    *
*          List panel) to determine the volume serial.                *
*                                                                     *
*      d.  BR must be authorized to read a VVDS or bypass ENQ.        *
*          If you plan to use these features, link BR into an         *
*          APF-authorized library under the name ISF$BRO, mark it     *
*          RENT, and make BR an alias.  If these conditions are       *
*          met, BR will be able to use the SDSF SVC to become         *
*          authorized.  For more information on the SDSF SVC,         *
*          please refer to the ISFSVC documentation file.             *
*                                                                     *
*---------------------------------------------------------------------*
*                                                                     *
*    Point-and-shoot Capabilities                                     *
*                                                                     *
*      If the user invokes BR without specifying a dsname,            *
*      BR tries to find a dsname on the screen, at the cursor         *
*      position; this technique, called "point-and-shoot",            *
*      is implemented here in two flavors:                            *
*                                                                     *
*      1. When it is invoked as an edit macro, BR uses standard       *
*         ISREDIT calls to retrieve the text under which the cursor   *
*         is currently located, scans the text for specific           *
*         delimiters, then extracts the data set name.                *
*                                                                     *
*      2. When it is invoked as a program or as a command,            *
*         BR extracts the screen image and cursor position            *
*         from the TLD, an ISPF control block that is not             *
*         part of the standard programming interface (this            *
*         technique may not work in future releases of ISPF).         *
*                                                                     *
*         Note that for this to work, it is imperative that BR        *
*         be invoked with MODE(FSCR).  Unfortunately, if you invoke   *
*         BR via the TSO command, the SELECT service invokes BR as    *
*         a command and uses MODE(LINE) as a default, which           *
*         prevents BR from retrieving the data set name.              *
*                                                                     *
*         So if you want use BR's point-and-shoot capabilities        *
*         outside of an EDIT session, you must use one of the         *
*         following options:                                          *
*                                                                     *
*           1. Instead of just TSO BR, type the following,            *
*              exactly as shown (no closing parenthesis)              *
*                                                                     *
*                TSO BR) MODE(FSCR                                    *
*                                                                     *
*              not something you want to type too often, but if       *
*              you set it up as a PFK, it's allright.                 *
*                                                                     *
*           2. Define BR as a command in the ISPF command table,      *
*              for example like this:                                 *
*                                                                     *
*                BR     0 SELECT PGM(BR) MODE(FSCR)                   *
*                                                                     *
*              then you can access BR just by typing BR on the        *
*              command line, on any screen.                           *
*                                                                     *
*           3. Write a simple REXX exec (or a CLIST) to invoke BR     *
*              with MODE(FSCR):                                       *
*                                                                     *
*                ARG dsn                                              *
*                "ISPEXEC SELECT CMD(BR" dsn ") MODE(FSCR)"           *
*                                                                     *
*              then you can access BR just by typing TSO %BR          *
*                                                                     *
*           4. Modify ISPCMDS to add MODE(FSCR) to the definition     *
*              of the TSO command                                     *
*                                                                     *
*                SELECT SUSPEND CMD(&ZPARM) MODE(FSCR)                *
*                                                                     *
*---------------------------------------------------------------------*
*                                                                     *
*    Assembly/link-edit information:                                  *
*                                                                     *
*         Environment:          MVS/XA 2.2 thru MVS/ESA 4.3           *
*                               ISPF/PDF V2R3 thru V3R5               *
*         AMODE:                31                                    *
*         RMODE:                Below                                 *
*         Non-IBM macro used:   STRING                                *
*         Processor:            Assembler H V2                        *
*         LKED Attributes:      RENT                                  *
*         Size of load-module:  7K                                    *
*                                                                     *
*    Author: Gilbert Saint-Flour (1987, 1990, 1992, 1993)             *
*                                                                     *
***********************************************************************
&VERMOD SETC 'R136'
*931231 136 Display program level on first screen
*931128 135 Change APPLID to ISR if invoked under SDSF
*931121 134 Use the SDSF SVC to open VVDS's and bypass ENQ
*931121 134 Use standard BROWSE to process PDSE's
*931107 133 Simplify processing of first record (ESDS/KSDS)
*931106 132 Do not ignore the member name, pass it to BROWSE
*931030 131 Issue ISRB603 message when VSAM data set is empty
*931015 130 Prevent loop if track is empty
*930327 122 Process data sets with null DSORG
*930322 121 Process RECFM=VS/VBS like RECFM=U
*930306 120 Add limited support for Linear data sets (first 64 K only)
*930130 110 Program made re-entrant, uses EXCP instead of BDAM
*930123 099 BR may now be invoked as a PGM or EDIT macro
*930123 099 Fixed a couple of bugs, added comments & TRACE macro
XBRVS    CSECT
XBRVS    AMODE 31
         B     12(,R15)                   SDSF SVC AMODE31 prefix  @134
         DC    A(0,ISFSVC+X'80000002')    SDSF SVC AMODE31 prefix  @134
         BALR  R15,0                                               @134
         SAVE  (14,12),,'VSAM/BDAM BROWSE &VERMOD'
         BALR  R11,0
         USING BASEADDR,R11
BASEADDR LA    R12,BASEADDR+4095
         USING BASEADDR+4095,R12
         GETMAIN R,LV=DYNAML,SP=1      ALLOCATE DYNAMIC STORAGE AREA
         ST    R13,4(,R1)
         ST    R1,8(,R13)
         LA    R14,72(,R1)             clear working storage
         LA    R15,CLEARLEN-72         clear working storage
         SLR   R1,R1                   clear working storage
         MVCL  R14,R0                  clear working storage
         LM    R13,R1,8(R13)           SET R13, RELOAD R14-R1
         LR    R9,R1                   save PARM/CPPL address
         USING DYNAM,R13
         STM   R11,R12,BASEREGS        for read routines
*
         L     R1,X'021C'              PSATOLD (my TCB)
         L     R1,0(,R1)               TCBRBP (my PRB)
         ICM   R1,B'0111',X'00D'(R1)   RBCDE1 (my CDE)
         ST    R1,OWNCDE               keep the address of my CDE
         USING CDENTRY,R1
         TM    CDATTR,CDMIN            IS this a minor CDE?
         BZ    *+8                     no, jump
         L     R1,CDXLMJP              yes, point to major CDE
         MVC   OWNNAME,CDNAME          save my true name
         DROP  R1
*----------------------------------------------------------------------
         MACRO                                                    macro
&NAME    ISPLINK &OPRNDS,&ERRET=                                  macro
         GBLB  &IHBSWA                                            macro
&IHBSWA  SETB  1                       VL=1                       macro
&NAME    LA    R1,TENWORDS             parm list                  macro
         IHBOPLST ,&OPRNDS,MF=(E,(1))                             macro
         L     R15,ISPLINK@            ISPLINK bootstrap
         BALR  R14,R15                 invoke ISPLINK
         AIF   (T'&ERRET EQ 'O').MEND                             macro
         LTR   R15,R15                 Any error?
         BNZ   &ERRET                  yes, jump
.MEND    MEND                                                     macro
*----------------------------------------------------------------------
         MACRO ,                                                        STRING
&NAME    STRING &INTO=,&PRINT=NOGEN                                     STRING
         AIF   ('&PRINT' EQ 'NOGEN').NOGEN                              STRING
         PUSH  PRINT                                                    STRING
         PRINT GEN                                                      STRING
.NOGEN   GBLA  &$_FIELD,&$_MAXBL                                        STRING
         GBLC  &$_FIELDS(9999)                                          STRING
         LCLA  &I,&J,&N                                                 STRING
         AIF   (N'&SYSLIST EQ 1 AND '&SYSLIST(1)' EQ 'FINAL_CALL'      XSTRING
               AND T'&INTO EQ 'O').GENL                                 STRING
&LABEL   SETC  'IHB&SYSNDX'            stem for local labels            STRING
&LQ      SETC  'L'''                   length attribute                 STRING
&NAME    BAS   R14,$STRING             Call @STRING sub-routine         STRING
         AIF   (N'&SYSLIST EQ 0).ERR1                                   STRING
         AIF   (T'&INTO EQ 'O').ERR2                                    STRING
         DC    AL2(&LABEL.P-*)                                          STRING
         AIF   (D'$STRING).LOCTR2                                       STRING
$LTORG   LOCTR ,                       ADDRESSABLE CONSTANTS            STRING
         CNOP  0,4                                                      STRING
$STRING  BALR  R15,0                   local base                       STRING
         L     R15,6(,R15)             routine address           00     STRING
         BSM   0,R15                   go there with amode31     04     STRING
         DC    A(@STRING+X'80000000')  routine address           06     STRING
$FARRTNE LOCTR ,                       far routines                     STRING
&$_MAXBL SETA  1                       At least one space          @301 STRING
.LOCTR2  ANOP                                                           STRING
$LITERAL LOCTR ,                       NON-ADDRESSABLE CONSTANTS        STRING
&TO1     SETC  '&INTO(1)'                                               STRING
&TO2     SETC  '&LQ&INTO'                                               STRING
         AIF   (N'&INTO EQ 1).PUNTO8            JUMP IF INTO=XXX        STRING
         AIF   ('&INTO(1)'(1,1) NE '(').PUNTO3  JUMP IF INTO=(XXX,44)   STRING
&TO1     SETC  '0&INTO(1)'                              INTO=((R3),44)  STRING
.PUNTO3  ANOP                                                           STRING
&TO2     SETC  '&INTO(2)'                               INTO=(XXX,LL)   STRING
         AIF   ('&INTO(2)'(1,1) NE '(').PUNTO8  JUMP IF INTO=(XXX,44)   STRING
&TO2     SETC  '0&INTO(2)'                              INTO=(XXX,(R1)) STRING
.PUNTO8  ANOP                                                           STRING
&LABEL.P DC    S(&TO1,&TO2)                                             STRING
         LCLB  &LAST,&BIN,&HEXA,&REG,&PACKED,&LEFT,&ZERO,&TRUNC         STRING
&I       SETA  1                                                        STRING
.LOOP1   ANOP                                                           STRING
&LAST    SETB  (&I EQ N'&SYSLIST)                LOOP                   STRING
         AIF   ('&SYSLIST(&I)'(1,1) EQ '''').LIT00                      STRING
&P1S     SETC  '&SYSLIST(&I,1)'                                         STRING
&P2L     SETC  '0'                     INPUT LENGTH                     STRING
&P3L     SETC  '0'                     OUTPUT LENGTH                    STRING
         AIF   ('&SYSLIST(&I)'(1,1) GE '0').FLD180 SPACES               STRING
         AIF   ('&SYSLIST(&I)' EQ '%TIME').FLD190 %TIME                 STRING
         AIF   ('&SYSLIST(&I,1)'(1,1) NE '(').FLD115 (R2)               STRING
         AIF   (T'&SYSLIST(&I,2) EQ 'O').FLD250                         STRING
&P1S     SETC  '0&SYSLIST(&I,1)'       CHANGE (R1) TO 0(R1)             STRING
.FLD115  ANOP                                                           STRING
         AIF   (T'&SYSLIST(&I,2) NE 'O').FLD200                         STRING
         AIF   (NOT D'&SYSLIST(&I)).FLD140                              STRING
&P2C     SETC  T'&SYSLIST(&I)                                           STRING
         AIF   ('&P2C' EQ 'F' OR '&P2C' EQ 'H' OR '&P2C' EQ 'P').FLD220 STRING
         AIF   ('&P2C' EQ 'G').FLD210  FL2                              STRING
.FLD140  ANOP                                                           STRING
&P2L     SETC  '&LQ&SYSLIST(&I,1)'     L'ABCDEF                         STRING
         AGO   .FLD300                                                  STRING
.FLD180  AIF   ('&SYSLIST(&I)'(K'&SYSLIST(&I),1) NE 'X').FLD800         STRING
&P1S     SETC  'BLANKS'                                                 STRING
&P2L     SETC  '&SYSLIST(&I)'(1,K'&SYSLIST(&I)-1) 12                    STRING
&L       SETA  &P2L                    NUMBER OF BLANKS                 STRING
         AIF   (&L LE &$_MAXBL).FLD800                                  STRING
&$_MAXBL SETA   &L                     MAX NUMBER OF BLANKS             STRING
         AGO   .FLD800                                                  STRING
.FLD190  ANOP                          %TIME                            STRING
&P1S     SETC  '1(14)'                 %TIME                            STRING
         AGO   .FLD800                                                  STRING
.FLD200  AIF   (T'&SYSLIST(&I,2) EQ 'O').FLD300 NO LENGTH SPECIFIED     STRING
&P2C     SETC  '&SYSLIST(&I,2)'                                         STRING
         AGO   .FLD220                                                  STRING
.FLD210  ANOP                                                           STRING
&L       SETA  L'&SYSLIST(&I)          T'&P1 = 'G'                      STRING
&P2C     SETC  'FL&L'                  T'&P1 = 'G'                      STRING
.FLD220  ANOP                                                           STRING
&P2L     SETC  '0&P2C'                 (R2) LENGTH                      STRING
         AIF   ('&P2C'(1,1) EQ '(').FLD300                              STRING
&P2L     SETC  '&P2C'                  3(R2) LENGTH                     STRING
         AIF   ('&P2C'(K'&P2C,1) EQ ')').FLD300                         STRING
&P2L     SETC  '0'                                                      STRING
&PACKED  SETB  ('&P2C' EQ 'P')                                          STRING
         AIF   (&PACKED).FLD300                                         STRING
&P2L     SETC  '1'                                                      STRING
         AIF   ('&P2C' EQ 'FL1').FLD240                                 STRING
&P2L     SETC  '3'                                                      STRING
         AIF   ('&P2C' EQ 'FL2' OR '&P2C' EQ 'H').FLD240                STRING
&P2L     SETC  '7'                                                      STRING
         AIF   ('&P2C' EQ 'FL3').FLD240                                 STRING
&P2L     SETC  '15'                                                     STRING
         AIF   ('&P2C' EQ 'F').FLD240                                   STRING
&P2L     SETC  '&P2C'                  IMMEDIATE LENGTH, FIELD          STRING
         AGO   .FLD300                                                  STRING
.FLD240  ANOP                          BINARY VARIABLE                  STRING
&BIN     SETB  1                                                        STRING
         AGO   .FLD300                                                  STRING
.FLD250  ANOP                          REGISTER CONTENT                 STRING
&REG     SETB  1                                                        STRING
.FLD300  AIF   (T'&SYSLIST(&I,3) EQ 'O').FLD800                         STRING
&HEXA    SETB  ('&SYSLIST(&I,3)' EQ 'X') HEXADECIMAL                    STRING
&TRUNC   SETB  ('&SYSLIST(&I,3)' EQ 'T') TRUNCATE                       STRING
         AIF   (&HEXA OR &TRUNC).FLD800                                 STRING
&P3C     SETC  '&SYSLIST(&I,3)'                                         STRING
&P3L     SETC  '250'                                                    STRING
         AIF   ('&P3C' EQ 'YY/MM/DD' AND &PACKED).FLD800                STRING
&P3L     SETC  '251'                                                    STRING
         AIF   ('&P3C' EQ 'DD/MM/YY' AND &PACKED).FLD800                STRING
&P3L     SETC  '252'                                                    STRING
         AIF   ('&P3C' EQ 'MM/DD/YY' AND &PACKED).FLD800                STRING
&P3L     SETC  '253'                                                    STRING
         AIF   ('&P3C' EQ 'YYMMDD'   AND &PACKED).FLD800                STRING
&P3L     SETC  '0'                                                      STRING
         AIF   (T'&SYSLIST(&I,2) NE 'N').FLD310                         STRING
         MNOTE 8,'EDIT PATTERN NOT ALLOWED WITH CHARACTER STRING'       STRING
.FLD310  AIF   ('&P3C'(1,1) EQ 'R').FLD318       DEFAULT                STRING
         AIF   ('&P3C'(1,1) EQ 'B').FLD318       DEFAULT                STRING
         AIF   ('&P3C'(1,1) NE 'L').FLD311                              STRING
&LEFT    SETB  1                                                        STRING
         AGO   .FLD318                                                  STRING
.FLD311  AIF   ('&P3C'(1,1) NE 'Z').FLD312                              STRING
&ZERO    SETB  1                                                        STRING
         AGO   .FLD318                                                  STRING
.FLD312  AIF   ('&P3C'(1,1) LT '0').FLD993                              STRING
&P3L     SETC  '&P3L'.'&P3C'(1,1)                                       STRING
.FLD318  ANOP                                                           STRING
&P3C     SETC  '&P3C'(2,K'&P3C-1)     STRIP OFF FIRST CHARACTER         STRING
         AIF   (K'&P3C GT 0).FLD310                                     STRING
.FLD800  AIF   ('&BIN&PACKED&REG' EQ '000').FLD810                      STRING
         AIF   (&LEFT OR '&P3L' NE '0').FLD810                          STRING
         AIF   (&HEXA).FLD810                                           STRING
&P3L     SETC  '7'                     DEFAULT OUTPUT LENGTH ((R3))     STRING
         AIF   (&REG).FLD810                                            STRING
&P3L     SETC  '3'                     DEFAULT OUTPUT LENGTH            STRING
         AIF   ('&P2C' EQ 'FL1').FLD810                                 STRING
&P3L     SETC  '5'                     DEFAULT OUTPUT LENGTH            STRING
         AIF   ('&P2C' EQ 'H' OR '&P2C' EQ 'FL2').FLD810                STRING
&P3L     SETC  '7'                     DEFAULT OUTPUT LENGTH            STRING
.FLD810  ANOP                                                           STRING
&FLAG    SETA  &LAST*128+&HEXA*8+&BIN*4+&PACKED*2+&REG*1                STRING
&LEN2    SETA  &TRUNC*128+&LEFT*128+&ZERO*64+&P3L                       STRING
         DC    S(&P1S,&P2L),AL1(&FLAG,&LEN2)                            STRING
&BIN     SETB  0                    RESET FLAGS                         STRING
&HEXA    SETB  0                    RESET FLAGS                         STRING
&REG     SETB  0                    RESET FLAGS                         STRING
&PACKED  SETB  0                    RESET FLAGS                         STRING
&LEFT    SETB  0                    RESET FLAGS                         STRING
&ZERO    SETB  0                    RESET FLAGS                         STRING
&TRUNC   SETB  0                    RESET FLAGS                         STRING
         AGO   .LIT99                                                   STRING
.FLD990  MNOTE 8,'MISSING OR INCORRECT PARAMETER'                       STRING
         AGO   .LIT99                                                   STRING
.FLD993  MNOTE 8,'THIRD SUBPARAMETER IS INVALID: ''&SYSLIST(&I,3)'''    STRING
         AGO   .LIT99                                                   STRING
.LIT00   AIF   (&$_FIELD EQ 0).LIT50                                    STRING
&N       SETA  1                                                        STRING
.LIT10   AIF   (&N GT &$_FIELD).LIT50                LOOP               STRING
&L       SETA  &N+1000                               LOOP               STRING
         AIF   ('&SYSLIST(&I)' EQ '&$_FIELDS(&N)').LIT80 LOOP           STRING
&N       SETA  &N+1                                  LOOP               STRING
         AGO   .LIT10                                LOOP               STRING
.LIT50   ANOP                                                           STRING
&$_FIELD SETA  &$_FIELD+1                                               STRING
&$_FIELDS(&$_FIELD) SETC '&SYSLIST(&I)'                                 STRING
&L       SETA  &$_FIELD+1000                                            STRING
.LIT80   ANOP                                                           STRING
&J       SETA  X'4000'+&LAST*X'8000'                                    STRING
         DC    AL2($LIT&L-*,&LQ.$LIT&L,&J)                              STRING
.LIT99   ANOP                                                           STRING
&I       SETA  1+&I                              LOOP                   STRING
         AIF   (&I LE N'&SYSLIST).LOOP1          LOOP                   STRING
&SYSLOC  LOCTR                                                          STRING
         AGO   .MEND                                                    STRING
.ERR1    MNOTE 12,'AT LEAST ONE INPUT FIELD MUST BE SPECIFIED'          STRING
         AGO   .MEND                                                    STRING
.ERR2    MNOTE 12,'INVALID OUTPUT AREA SPECIFICATION'                   STRING
         AGO   .MEND                                                    STRING
.GENL    AIF   (&$_FIELD EQ 0).GENL3                                    STRING
$LITERAL LOCTR                                                          STRING
.GENL2   ANOP                                LOOP                       STRING
&N       SETA  &N+1                          LOOP                       STRING
&I       SETA  &N+1000                       LOOP                       STRING
$LIT&I   DC    C&$_FIELDS(&N)                                           STRING
         AIF   (&N LT &$_FIELD).GENL2        LOOP                       STRING
.GENL3   ANOP                                                           STRING
@STRING  CSECT                                                          STRING
@STRING  RMODE ANY                                                      STRING
         SAVE  (14,6),,'@STRING 3.00 '                                  STRING
         USING @STRING,R15                                              STRING
         USING @STRSAVE,R13                                             STRING
         LR    R6,R14                  KEEP ADDRESS OF PARMLIST OFFSET  STRING
         USING @STRPARM,R6                                              STRING
         SLR   R0,R0                                                    STRING
         ICM   R0,B'0011',0(R6)        PICK UP PARM LIST OFFSET         STRING
         ALR   R6,R0                   GET PARM LIST ADDRESS            STRING
         LA    R2,@STRSCON             RESOLVE FIRST SCON (ADDR)        STRING
         BAL   R14,@STRS2A             GET ADDRESS IN R2                STRING
         LR    R4,R2                   KEEP ADDRESS OF "INTO" FIELD     STRING
         OI    0(R4),0                 get clean S0C4 if address is bad STRING
         LA    R2,@STRFLEN             RESOLVE SECOND S-CON (LEN)       STRING
         BAL   R14,@STRS2A             GET LENGTH IN R2                 STRING
         LR    R5,R2                   KEEP LENGTH OF "INTO" FIELD      STRING
         CR    R5,R4                   end address?                     STRING
         BL    @STR282                 NO, JUMP                         STRING
         SR    R5,R4                   CALCULATE LENGTH                 STRING
@STR282  ST    R4,8(,R13)              SAVE ADDRESS FOR LATER           STRING
         LA    R6,@STRFLAG             POINT TO 1ST FIELD DESC          STRING
@STR310  CLI   @STRSCON,X'E0'          CHECK FOR %TIME                  STRING
         BE    @STR388T                JUMP IF %TIME                    STRING
         TM    @STRFLAG,@STRLIT        IS THIS A LITERAL?               STRING
         BO    @STR388L                yes, process it                  STRING
         TM    @STRFLAG,@STRREG        CHECK CONVERSION TYPE            STRING
         BO    @STR335                 JUMP IF REGISTER                 STRING
         LA    R2,@STRFLEN             RESOLVE SECOND S-CON (LEN)       STRING
         BAL   R14,@STRS2A             GET LENGTH IN R2                 STRING
         LR    R3,R2                   KEEP LENGTH                      STRING
         LA    R2,@STRSCON             RESOLVE FIRST SCON (ADDR)        STRING
         BAL   R14,@STRS2A             GET ADDRESS IN R2                STRING
         CLI   0(R2),0                 get clean S0C4 if address is bad STRING
         CR    R3,R2                   end address?                     STRING
         BL    @STR312                 NO, JUMP                         STRING
         LA    R0,X'0080'              PSA address              @921221 STRING
         CLR   R2,R0                   PSA reference?           @921221 STRING
         BL    @STR312                 yes, jump                @921221 STRING
         SR    R3,R2                   CALCULATE LENGTH                 STRING
@STR312  TM    @STRFLAG,@STRPACK       CHECK CONVERSION TYPE            STRING
         BO    @STR330                 JUMP IF PACKED FIELD             STRING
         TM    @STRFLAG,@STRHEX        CHECK CONVERSION TYPE            STRING
         BO    @STR361                 JUMP IF HEX STRING               STRING
         TM    @STRFLAG,@STRBIN        CHECK CONVERSION TYPE            STRING
         BZ    @STR370                 JUMP IF NO CONVERSION REQ'D      STRING
@STR320  SLR   R0,R0                                                    STRING
         EX    R3,@STRICM              LOAD THE BINARY VARIABLE         STRING
         B     @STR340                 EDIT R0 VALUE                    STRING
@STRICM  ICM   R0,*-*,0(R2)            LOAD A BINARY VARIABLE           STRING
@STR330  LR    R3,R2                   FIRST BYTE OF PACKED FIELD       STRING
         BALR  R14,0                                                    STRING
         TM    0(R3),X'0C'             IS THIS THE SIGN BYTE?           STRING
         LA    R3,1(,R3)                (NEXT BYTE)                     STRING
         BNOR  R14                     NO, LOOP MORE                    STRING
         SLR   R3,R2                   GET LENGTH OF PACKED FIELD       STRING
         BCTR  R3,0                                                     STRING
         EX    R3,@STRZAP              EXECUTE ZAP                      STRING
         CLI   @STRLEN2,250            JULIAN-TO-YYMMDD CONV?           STRING
         BL    @STR341                 no, edit DWD                     STRING
         B     @STR380                 yes, convert it                  STRING
@STR335  LH    R1,@STRSCON             REG NUMBER IN BITS 28-31         STRING
         EX    R1,@STR335S             copy R7-R13 INTO r0              STRING
         CLI   @STRSCON+1,R7           IS THIS R7-R13?                  STRING
         BNL   @STR336                 yes, jump                        STRING
         SLL   R1,2                    R1= 000000BB BASE * 4            STRING
         L     R0,20(R1,R13)           PICK UP VALUE FOR R0-R6          STRING
@STR336  TM    @STRFLAG,@STRHEX        CHECK CONVERSION TYPE            STRING
         BO    @STR360                 JUMP IF HEX STRING               STRING
@STR340  CVD   R0,@STRDWD              CONVERT VALUE TO DECIMAL         STRING
@STR341  IC    R0,@STRLEN2             OUTPUT LENGTH                    STRING
         LA    R3,X'003F'              MASK FOR "AND"                   STRING
         NR    R3,R0                   OUTPUT LENGTH                    STRING
         MVC   @STRWK16(16),@STRMASK   EDIT MASK                        STRING
         TM    @STRLEN2,@STRLEFT       CHECK JUSTIFICATION              STRING
         BO    @STR350                 JUMP IF LEFT JUSTIFICATION       STRING
         TM    @STRLEN2,@STRZERO       leading zeroes req'd?            STRING
         BNO   @STR344                 no, jump                         STRING
         MVI   @STRWK16,C'0'           yes, change X'40' to C'0'        STRING
@STR344  ED    @STRWK16(16),@STRDWD    ZONED DECIMAL                    STRING
         LA    R2,@STRWK16+16          FIRST POSITION AFTER STRING      STRING
         SR    R2,R3                   FIRST STRING POSITION            STRING
         B     @STR390                 MOVE STRING TO OUTPUT LINE       STRING
@STR350  LA    R1,@STRWK16+15          PREVENT BAD R1                   STRING
         EDMK  @STRWK16(16),@STRDWD    ZONED DECIMAL                    STRING
         LR    R2,R1                   FIRST STRING POSITION            STRING
         LTR   R3,R3                   CHECK OUTPUT LENGTH              STRING
         BNZ   @STR353                 JUMP IF NOT ZERO                 STRING
         LA    R3,@STRWK16+16          FIRST POSITION AFTER STRING      STRING
         SR    R3,R2                   COMPUTE STRING LENGTH            STRING
         B     @STR390                 MOVE FIELD TO OUTPUT LINE        STRING
@STR353  CR    R5,R3                   SUBTRACT LEN FROM REMAINING LEN  STRING
         BNL   @STR354                 JUMP IF LARGE ENOUGH             STRING
         LR    R3,R5                   TRUNCATE TO REMAINING LENGTH     STRING
@STR354  SR    R5,R3                   COMPUTE REMAINING LENGTH         STRING
         LR    R0,R4                   POINTER IN OUTPUT LINE           STRING
         LR    R1,R3                   LENGTH WITH PADDING              STRING
         LA    R3,@STRWK16+16          FIRST POSITION AFTER STRING      STRING
         SR    R3,R2                   COMPUTE STRING LENGTH            STRING
         B     @STR392                 MOVE FIELD TO OUTPUT LINE        STRING
@STR360  STCM  R0,B'1111',@STRDWD-1    STORE IT FOR CVD/UNPK            STRING
         LA    R3,8                    output LENGTH                    STRING
         B     @STR363                                                  STRING
@STR361  LTR   R3,R3                   ZERO LENGTH?                     STRING
         BZ    @STR398                 YES, IGNORE IT                   STRING
         LA    R0,8                    MAX LENGTH                       STRING
         CLR   R3,R0                   CHECK LENGTH                     STRING
         BNH   @STR362                 JUMP IF LE 8                     STRING
         LR    R3,R0                   TRUNCATE TO MAXIMUM LENGTH       STRING
@STR362  LR    R1,R3                   input length                     STRING
         BCTR  R1,0                                                     STRING
         EX    R1,@STR360M             MOVE DATA TO SAFE STORAGE        STRING
         ALR   R3,R3                   output length                    STRING
@STR363  LA    R2,@STRWK16             work area                        STRING
         UNPK  0(9,R2),@STRDWD-1(5)    EXPAND SOURCE BYTES FOR "TR"     STRING
         UNPK  8(9,R2),@STRDWD+3(5)    EXPAND SOURCE BYTES FOR "TR"     STRING
         TR    0(16,R2),@STRHEXT-240   =C'0123456789ABCDEF'             STRING
@STR370  CLI   @STRLEN2,@STRLEFT       CHECK JUSTIFICATION, OUTPUT LEN  STRING
         BNE   @STR390                 NO STRING TRUNCATION, JUMP       STRING
         LA    R1,0(R3,R2)             FIRST BYTE AFTER FIELD           STRING
@STR372  BCTR  R1,0                    DOWN 1 BYTE                 LOOP STRING
         CLI   0(R1),C' '              IS IT A SPACE ?             LOOP STRING
         BNE   @STR390                 LAST NON-BLANK BYTE         LOOP STRING
         BCT   R3,@STR372              LOOP UNTIL 1ST NON-BLANK    LOOP STRING
         B     @STR398                 BLANK FIELD, DO NOT EDIT         STRING
@STR335S LR    R0,*-*                  COPY VALUE FOR EDIT              STRING
@STRZAP  ZAP   @STRDWD,0(*-*,R2)       MOVE TO @STRDWD                  STRING
@STR360M MVC   @STRDWD-1(*-*),0(R2)    PREVENT S0C4 IN UNPK             STRING
@STRHEXT DC    C'0123456789ABCDEF'                                      STRING
@STRMASK DC    X'4020202020202020,2020202020202120'                     STRING
@STR380  ZAP   @STRDWD,@STRDWD         DATE=0000000?               @JDT STRING
         BZ    @STR380Z                YES, JUMP                   @JDT STRING
         MVO   @STRDWD+0(2),@STRDWD+4+1(1)           0YY00000,00YY0DDC  STRING
         MVI   @STRDWD+2,X'0C'                       0YY00C00,00YY0DDC  STRING
         BAL   R1,@STR380T+2*13        TABLE OF MONTHS (NUMBER OF DAYS) STRING
@STR380T DC    P'59,31,29,31,30,31,30,31,31,30,31,30,31'                STRING
         TM    @STRDWD+4+1,X'01'       ODD YEARS                        STRING
         BO    @STR380N                  AREN'T LEAP YEARS              STRING
         TM    @STRDWD+4+1,X'12'       ZEROES IN 1980, ALL ONES IN 1992 STRING
         BNM   @STR380L                MIXED IN 1982/1990               STRING
@STR380N CP    @STRDWD+4+2(2),@STR380T ARE WE PAST FEB 28 (DDD>59) ?    STRING
         BNH   @STR380L                NO, JUMP                         STRING
         AP    @STRDWD+4+2(2),@STR380T+3(1) ADD 1 (FROM 31) TO DDD      STRING
@STR380L AP    @STRDWD+1(2),@STR380T+3(1)   ADD 1 (FROM 31) TO MONTH    STRING
         LA    R1,2(,R1)               NEXT ENTRY IN "MONTHS" TABLE     STRING
         SP    @STRDWD+4+2(2),0(2,R1)  SUB DAYS-IN-MONTH FROM DDD       STRING
         BP    @STR380L                                                 STRING
         AP    @STRDWD+4+2(2),0(2,R1)  UNDO LAST "SP" INSTRUCTION       STRING
         UNPK  @STRDWD+0(4),@STRDWD+0(3) FYFYFMCM,00YY0DDC              STRING
         UNPK  @STRDWD+4(2),@STRDWD+6(2) FYFYFMCM,FDCD0DDC              STRING
         MVZ   @STRDWD+3(3),@STRDWD+2    FYFYFMFM,FDFD0DDC              STRING
         MVI   @STRDWD+6,C'/'          SEPARATOR                        STRING
         LA    R0,250                  MASK FOR 'NR'                    STRING
         SLR   R1,R1                                                    STRING
         IC    R1,@STRLEN2             250-255                          STRING
         SLR   R1,R0                   000-005                          STRING
         LA    R0,1+8                  L'@STR380W                       STRING
         MR    R0,R0                   COMPUTE OFFSET                   STRING
         LA    R1,@STR380W(R1)         entry in "TR" MASK TABLE         STRING
         LA    R2,@STRWK16             WORK AREA                        STRING
         SLR   R3,R3                                                    STRING
         IC    R3,0(,R1)               LENGTH OF DATE (6 OR 8)          STRING
         MVC   0(8,R2),1(R1)           MOVE CORRESPONDING MASK          STRING
         TR    0(8,R2),@STRDWD         CONVERT DATE TO THE RIGHT FORMAT STRING
         B     @STR390                 MOVE FIELD TO OUTPUT LINE        STRING
@STR380W DC    AL1(8,0,1,6,2,3,6,4,5)  YY/MM/DD   250                   STRING
         DC    AL1(8,4,5,6,2,3,6,0,1)  DD/MM/YY   251                   STRING
         DC    AL1(8,2,3,6,4,5,6,0,1)  MM/DD/YY   252                   STRING
         DC    AL1(6,0,1,2,3,4,5,6,6)  YYMMDD//   253                   STRING
@STR380Z BAL   R2,*+4+8                WORK AREA                        STRING
         DC    8C' '                   RETURN SPACES                    STRING
         LA    R3,008                  LENGTH OF 'YY/MM/DD'             STRING
         CLI   @STRLEN2,253            YY/MM/DD OK?                     STRING
         BL    @STR390                 YES, JUMP                        STRING
         LA    R3,006                  LENGTH OF 'YYMMDD'               STRING
         B     @STR390                                                  STRING
@STR388T TIME  DEC                     GET HHMMSSHH                     STRING
         L     R15,16(,R13)            RESTORE BASE REG                 STRING
         ST    R0,@STRDWD              STORE HHMMSSHH                   STRING
         MVC   @STRWK16(13),@STRTIME   MOVE EDIT MASK                   STRING
         ED    @STRWK16(13),@STRDWD    EDIT HH:MM:SS:HH                 STRING
         LA    R2,@STRWK16+1           WORK AREA                        STRING
         LA    R3,12                   HH:MM:SS:HH+ SPACE               STRING
         B     @STR390                                                  STRING
@STRTIME DC    X'4021207A20207A20207A20204000'    0X.XX.XX.XX           STRING
@STR388L SLR   R2,R2                                                    STRING
         ICM   R2,B'0011',@STRSCON     LOAD LITERAL OFFSET              STRING
         LA    R2,@STRSCON(R2)         CONVERT OFFSET TO FULL ADDRESS   STRING
         LH    R3,@STRFLEN             GET LITERAL LENGTH               STRING
@STR390  CR    R5,R3                   COMPARE REM. LEN TO CURR LEN     STRING
         BNM   @STR391                 JUMP IF POSITIVE OR ZERO         STRING
         LR    R3,R5                   TRUNCATE TO REMAINING LENGTH     STRING
@STR391  SR    R5,R3                   COMPUTE REMAINING LENGTH         STRING
         LR    R0,R4                   POINTER IN OUTPUT LINE           STRING
         LR    R1,R3                   PASS REMAINING LENGTH            STRING
@STR392  ICM   R3,B'1000',@STRPAD      PAD WITH BLANKS                  STRING
         MVCL  R0,R2                   MOVE FIELD TO OUTPUT LINE        STRING
         LR    R4,R0                   NEW POINTER IN OUTPUT LINE       STRING
@STR398  TM    @STRFLAG,@STRLAST       TEST LAST-ENTRY INDICATOR        STRING
         LA    R6,@STRNEXT             BUMP UP TO NEXT ENTRY            STRING
         BNO   @STR310                 PROCESS NEXT ENTRY               STRING
         LA    R14,2                   increment                        STRING
         AL    R14,12(,R13)            RETURN ADDRESS                   STRING
         ICM   R1,B'1111',@STRPAD      SET UP R1 FOR PADDING            STRING
         LR    R15,R4                  CURRENT POINTER IN OUTPUT FIELD  STRING
         SL    R15,8(,R13)             CALCULATE LENGTH USED            STRING
         MVCL  R4,R0                   PAD WITH BLANKS                  STRING
         LM    R0,R6,20(R13)           RESTORE WORK REGISTERS           STRING
         STM   R7,R12,48(R13)          MAKE SAVE AREA LOOK NORMAL       STRING
         OI    15(R13),1               SIMULATE "T" OPTION OF RETURN    STRING
         BSM   0,R14                   RETURN TO CALLER                 STRING
@STRS2A  SLR   R0,R0                                                    STRING
         ICM   R0,B'0011',0(R2)        R0 = 0000BDDD                    STRING
         SRDL  R0,12                   R0 = 0000000B, R1= DDD.....      STRING
         SRL   R1,20                   R1 = 00000DDD  (DISPLACEMENT)    STRING
         CLI   0(R2),R7*16             R7-R13?                          STRING
         BL    @STRS2A5                NO, JUMP                         STRING
         LR    R2,R0                   R2= 0000000B (BASE REG NUMBER)   STRING
         EX    R2,@STRS2A4             ADD BASE REG VALUE TO DISPL      STRING
         B     @STRS2A7                clean up address                 STRING
@STRS2A4 LA    R2,0(*-*,R1)            ADD BASE REG VALUE TO DISPL      STRING
@STRS2A5 LTR   R2,R0                   is R0 the base reg?              STRING
         BNZ   @STRS2A6                no, jump                         STRING
         LTR   R2,R1                   IS THIS A PSA ADDRESS?           STRING
         BNZR  R14                     yes, goback                      STRING
@STRS2A6 SLL   R2,2                    R2= 000000BB BASE * 4            STRING
         L     R2,20(R2,R13)           PICK UP BASE REG VALUE           STRING
         LA    R2,0(R2,R1)             ADD BASE REG VALUE TO DISPL      STRING
@STRS2A7 TM    12(R13),X'80'           caller's in amode31?             STRING
         BOR   R14                     yes, goback                      STRING
         ICM   R2,B'1000',@STRPAD+1    no, zero hi-order byte           STRING
         BR    R14                                                      STRING
@STRPAD  DC    X'40,000000'            PADDING                          STRING
@STRSAVE DSECT                         24-BYTE WORK AREA                STRING
         DS    A(0,@STRSAVE,@STRSAVE,14,15,0,1,2,3,4,5,6)               STRING
@STRWK16 DS    F'7,8,9,10'             WORK AREA                        STRING
@STRDWD  DS    D'1112'                 WORK AREA                        STRING
@STRPARM DSECT                                                          STRING
@STRSCON DS    S                   +0  FIELD ADDRESS                    STRING
@STRFLEN DS    S                   +2  FIELD LENGTH                     STRING
@STRFLAG DS    B                   +4  FORMAT, FLAGS                    STRING
@STRLAST EQU   X'80'                     LAST ENTRY                     STRING
@STRLIT  EQU   X'40'                     LITERAL, @STRSCON IS AN OFFSET STRING
@STRJDT  EQU   X'20'                     JULIAN TO YYMMDD               STRING
@STRHEX  EQU   X'08'                     HEXADECIMAL                    STRING
@STRBIN  EQU   X'04'                     BINARY                         STRING
@STRPACK EQU   X'02'                     PACKED                         STRING
@STRREG  EQU   X'01'                     REGISTER                       STRING
@STRLEN2 DS    B                   +5  FORMAT, OUTPUT LENGTH            STRING
@STRLEFT EQU   X'80'                     LEFT JUSTIFICATION             STRING
@STRZERO EQU   X'40'                     LEADING ZEROES                 STRING
@STRNEXT EQU   *                   +6                                   STRING
$LTORG   LOCTR                                                          STRING
         AIF   (D'BLANKS).MEND                                          STRING
BLANKS   DC    &$_MAXBL.C' '       A BUNCH OF BLANKS                    STRING
.MEND    AIF   ('&PRINT' EQ 'NOGEN').MEND99                             STRING
         POP   PRINT                                                    STRING
.MEND99  MEND                                                           STRING
*----------------------------------------------------------------------
         LOAD  EP=ISPLINK
         ST    R0,ISPLINK@             save ISPLINK address
         LOAD  EP=ISPEXEC
         ST    R0,ISPEXEC@             save ISPEXEC address
         ISPLINK (=C'CONTROL ',=C'ERRORS ',=C'RETURN ')
         LTR   R15,R15                 ISPF active?
         BNZ   QUIT16                  no, quit
*
         LA    R0,L'ZAPPLID            max length of dsname
         ST    R0,DWD                  max length of dsname
         ISPLINK (=C'VCOPY ',=C'(ZAPPLID)',DWD,ZAPPLID,=C'MOVE '),     X
               ERRET=QUIT16
*
         STRING 09X,'BR Command &VERMOD',INTO=MESSAGE
         ISPLINK (=C'VREPLACE',=C'ZEDSMSG ',=A(L'MESSAGE),MESSAGE)
         ISPLINK (=C'VREPLACE',=C'ZEDLMSG ',=A(L'MESSAGE),MESSAGE)
         ISPLINK (=C'SETMSG ',=C'ISRZ000 ')
*
         ICM   R3,B'1111',0(R9)        INVOKED AS A PGM?
         BM    PGM000                  Yes, jump
         USING CPPL,R9                 SET UP ADDRESSABILITY TO CPPL
*
*        If no dsname has been specified, scan the screen buffer
*        at the cursor position to find a dsname
*
         L     R1,CPPLECT              ENVIRONMENT CONTROL TABLE
         TM    ECTSWS-ECT(R1),ECTNOPD  any operand specified?
         BZ    CALLPARS                yes, scan buffer
         BAL   R14,CURSOR           <- scan screen buffer for a dsname
         LTR   R0,R0                   dsname found?
         BNZ   MOVEDSN                 yes, used it
***********************************************************************
*                                                                     *
*        BUILD PARSE PARM LIST, CALL IKJPARS                          *
*                                                                     *
***********************************************************************
CALLPARS LA    R1,TENWORDS             POINT TO PPL
         USING PPL,R1                  SET UP ADDRESSABILITY TO PPL
         L     R0,CPPLUPT              USER PROFILE TABLE
         ST    R0,PPLUPT               STORE UPT ADDRESS IN PPL
         L     R0,CPPLECT              ENVIRONMENT CONTROL TABLE
         ST    R0,PPLECT               STORE ECT IN PPL
         LA    R0,ECB1                 EVENT CONTROL BLOCK
         ST    R0,PPLECB               STORE ECB IN PPL
         MVC   PPLPCL,=A(PCLCSECT)     STORE PCL ADDR INTO PPL
         LA    R0,DYNANSWR             ANSWER AREA
         ST    R0,PPLANS               STORE ANSWER ADDRESS IN PPL
         L     R0,CPPLCBUF             COMMAND BUFFER
         ST    R0,PPLCBUF              STORE BUFFER ADDRESS IN PPL
         LA    R14,=C'IKJPARS '        NAME OF PARSE ROUTINE
         L     R15,0016                CVTPTR
         L     R15,008(,R15)           CVTLINK
         STM   R14,R15,DWD             BUILD LINK PARMLIST
         LINK  SF=(E,DWD)              CALL THE PARSE ROUTINE
         LTR   R7,R15                  CHECK FOR ZERO PARSE RETURN
         BNZ   QUIT16                  GO AWAY UPSET
         DROP  R1,R9                   PPL, CPPL
***********************************************************************
*                                                                     *
*        PROCESS PARSED PARAMETERS                                    *
*                                                                     *
***********************************************************************
PARSE100 L     R2,DYNANSWR             LOAD PDL ADDRESS
         USING IKJPARMD,R2             SET UP ADDRESSABILITY TO PDL
         LA    R14,DSNAME              44-BYTE AREA
         LA    R15,L'DSNAME            GET LENGTH
         L     R0,DSNPCE               GET ADDRESS
         LH    R1,DSNPCE+4             ACTUAL LENGTH OF DSN
         ICM   R1,B'1000',BLANKS       PADDING
         MVCL  R14,R0                  MOVE DATA SET NAME
         LA    R14,MEMBER              8-BYTE AREA
         LA    R15,L'MEMBER            GET LENGTH
         L     R0,DSNPCE+8             GET ADDRESS
         LH    R1,DSNPCE+8+4           ACTUAL LENGTH OF member name
         ICM   R1,B'1000',BLANKS       PADDING
         MVCL  R14,R0                  MOVE DATA SET NAME
*TPUT DSNAME,L'DSNAME
*TPUT MEMBER,L'MEMBER
         TM    VOLPCE2+6,X'80'         CHECK VOLSER SUBPARAMETER
         BZ    PARSE300                JUMP IF NO SUBPARAM SPECIFIED
         L     R1,VOLPCE2+00           GET SUBPARAM ADDRESS
         MVC   VOLSER,0(R1)            MOVE 6-BYTE VOLUME SERIAL
         MVI   VOLGIVEN,C'Y'           VOLSER given by user
         B     PARSE999                exit
         DROP  R2
*
*        retrieve the volser from PDF 3.4 panel
*
PARSE300 LA    R0,L'FULLDSN            length
         ST    R0,DWD
         ISPLINK (=C'VCOPY ',=C'ZDLDSN ',DWD,FULLDSN,=C'MOVE '),       X
               ERRET=PARSE999          VCOPY FAILED, QUIT
         CLC   DSNAME,FULLDSN          same data set name?
         BNE   PARSE999                no, exit
         LA    R0,L'ZDLDSORG           length
         ST    R0,DWD
         ISPLINK (=C'VCOPY ',=C'ZDLDSORG ',DWD,ZDLDSORG,=C'MOVE '),    X
               ERRET=PARSE999          VCOPY FAILED, QUIT
         CLC   =C'VS',ZDLDSORG         vsam data set?
         BE    PARSE999                yes, exit
         LA    R0,L'VOLSER             length
         ST    R0,DWD
         ISPLINK (=C'VCOPY ',=C'ZDLVOL ',DWD,VOLSER,=C'MOVE '),        X
               ERRET=PARSE999          VCOPY FAILED, QUIT
         MVI   VOLGIVEN,C'Y'           VOLSER given by user (faked)
*
PARSE999 IKJRLSA DYNANSWR              FREE PDL
         B     ALLOCATE
*----------------------------------------------------------------------
         MACRO
&LABEL   TRACE ,
&LABEL   LA    R1,=C'&LABEL'
&L       SETA  K'&LABEL
         LA    R0,&L                   length of literal
         SVC   93                      issue TPUT SVC
         MEND
         MACRO
&LABEL   TRACE ,                       (dummy macro)
         AIF   (T'&LABEL EQ 'O').MEND
&LABEL   DS    0H                      define label
.MEND    MEND
***********************************************************************
*                                                                     *
*        I've been invoked as a pgm, check if it is an edit macro     *
*                                                                     *
***********************************************************************
PGM000   TRACE
         LH    R0,0(,R3)               dsn length
         LA    R1,2(,R3)               dsn address
         LTR   R0,R0                   DO WE HAVE A PARM"
         BH    MOVEDSN                 yes, jump
         ISPLINK (=C'ISREDIT ',=F'11',=C'MACRO (VAR)'),ERRET=PGM500
*
*        BR has been invoked as an edit macro.  The first
*        parameter is used as dsname if a parm has been specified;
*        else, the dsname is picked up at the current cursor
*        location using standard edit macro calls.
*
PGM100   LA    R0,L'MESSAGE            max length of dsname
         ST    R0,DWD                  max length of dsname
         ISPLINK (=C'VCOPY ',=C'(VAR)',DWD,MESSAGE,=C'MOVE '),         X
               ERRET=PGM500
         ICM   R0,B'1111',DWD          Actual length of dsname
         BZ    PGM110                  no dsname, jump
         L     R1,X'21C'               PSATOLD (my TCB)
         L     R1,012(,R1)             TCBTIO (my TIOT)
         STRING ((R1),8,T),'.',MESSAGE,INTO=DSNAME
         CLI   MESSAGE,C''''           quoted dsn?
         BNE   ALLOCATE                No, use 'userid.dsname'
         BCTR  R0,0
         BCTR  R0,0
         STRING (MESSAGE+1,(R0)),INTO=DSNAME
         B     ALLOCATE                There is a dsname, ise it
*
*        get the dsname at cursor location
*
PGM110   TRACE
         ISPLINK (=C'ISREDIT ',=F'19',=C'(LINE,COL) = CURSOR'),        X
               ERRET=PGM500
PGM110T  TRACE
         LA    R0,3                    max length of variable
         ST    R0,DWD                  max length of variable
         ISPLINK (=C'VCOPY ',=C'(COL)',DWD,MESSAGE,=C'MOVE '),         X
               ERRET=PGM500
PGM120T  TRACE
         PACK  DWD,MESSAGE(3)          pack cursor position
         CVB   R5,DWD                  cursor position
         LTR   R5,R5                   cursor in command line?
         BZ    PGM500                  yes, jump
         ISPLINK (=C'ISREDIT ',=F'18',=C'(VAR) = LINE .ZCSR'),         X
               ERRET=PGM500
PGM125T  TRACE
         LA    R0,255                  max length of line
         ST    R0,DWD                  max length of line
         LA    R4,BUFFER               work area
         ISPLINK (=C'VCOPY ',=C'(VAR)',DWD,(R4),=C'MOVE '),ERRET=PGM500
*
         BCTR  R5,0                    R5=0 if cursor is in pos 1
         BAL   R14,CURSOR21            scan line to extract dsname
         LTR   R0,R0                   dsname found?
         BP    MOVEDSN                 yes, process it
*
*        This is not an edit macro or no dsname has been found
*        at the cursor position; scan the sceen buffer for a dsn
*
PGM500   BAL   R14,CURSOR           <- scan screen buffer for a dsname
         LTR   R0,R0                   dsname found?
         BNZ   MOVEDSN                 yes, used it
*
*        No dsname is available, re-invoke BR as a command,
*        which will prompt the user for the dsname
*
PGM600   STRING 'CMD(',OWNNAME,') MODE(FSCR) NEWAPPL(',ZAPPLID,')',    X
               INTO=MESSAGE
         ISPLINK (=C'SELECT ',=A(L'MESSAGE),MESSAGE)
         LR    R7,R15                  SAVE RETURN CODE FROM ISPLINK
         B     QUIT90
S99T15   DC    X'0015,0001,0008',C'SYSALLDA'
***********************************************************************
*                                                                     *
*        Allocate the data set                                        *
*                                                                     *
***********************************************************************
MOVEDSN  STRING ((R1),(R0)),INTO=DSNAME  move dsname
*
ALLOCATE TRACE
         LA    R0,L'DSNAME             Convert to uppercase
         LA    R1,DSNAME               Convert to uppercase
CAPSON   OI    0(R1),C' '              Convert to uppercase
         LA    R1,1(,R1)               Convert to uppercase
         BCT   R0,CAPSON               Convert to uppercase
*
         STRING '''',(DSNAME,,T),'''',INTO=FULLDSN
         CLI   MEMBER,C' '             do we have a member name?
         BNH   ALLOC100                no, jump
         STRING '''',(DSNAME,,T),'(',(MEMBER,,T),')''',INTO=FULLDSN
*----------------------------------------------------------------------
*                                                                     *
*        If ZAPPLID=ISF, re-invoke myself with NEWAPPL(ISR)           *
*                                                                     *
*----------------------------------------------------------------------
ALLOC100 CLC   =C'ISF ',ZAPPLID        invoked from SDSF?
         BNE   ALLOC200                no, jump
         CLI   VOLGIVEN,C'Y'           do we have a volser?
         BE    ALLOC120                no, jump
         STRING 'CMD(',OWNNAME,1X,(FULLDSN,,T),                        X
               ') MODE(FSCR) NEWAPPL(ISR)',                            X
               INTO=MESSAGE
         B     ALLOC180
ALLOC120 STRING 'CMD(',OWNNAME,1X,(FULLDSN,,T),                        X
               ') VOL(',VOLSER,                                        X
               ') MODE(FSCR) NEWAPPL(ISR)',                            X
               INTO=MESSAGE
ALLOC180 ISPLINK (=C'SELECT ',=A(L'MESSAGE),MESSAGE)
         LR    R7,R15                  SAVE RETURN CODE FROM ISPLINK
         B     QUIT90                  and exit
*----------------------------------------------------------------------
*                                                                     *
*        Allocate the data set                                        *
*                                                                     *
*----------------------------------------------------------------------
*LOAD EP=GSFDEBUG
*LR R15,R0
*BASSM R14,R15
ALLOC200 LA    R0,SAVERRTN             DDATA FOR BRIF
         ST    R0,DDATA                DDATA FOR BRIF
*
         LA    R14,ACB1_M              point to ACB, RPL, DCB models
         LA    R15,MODELS_L            length for move
         LA    R0,IFGACB               dynamic storage area
         LR    R1,R15                  length
         MVCL  R0,R14                  move models to dynamic storage
*
         IHBOPLST ,(S99T1,             return ddname                   X
               S99T2,                  dsname                          X
               =X'0004,0001,0001,08',  DISP=SHR                        X
               S99T15,                 UNIT=SYSALLDA                   X
               S99T57,                 return dsorg                    X
               S99T5D),                return volser                   X
               MF=(E,S99TUPTR)
         ST    R1,S99RB+8              Text unit pointers
         MVC   S99RB,=X'14,01,2000'
         MVC   S99T1(6),=X'0055,0001,0008'    return ddname
         MVC   S99T2(6),=X'0002,0001,002C'    dsname
         MVC   S99T57(6),=X'0057,0001,0002'   return dsorg
         MVC   S99T5D(6),=X'005D,0001,0006'   RETURN VOLSER
         CLI   VOLGIVEN,C'Y'           VOLSER given by user ?
         BNE   ALLOC600                no, jump
         MVI   S99T5D+1,X'10'          yes, use volser
*
*        Allocate the data set
*
ALLOC600 LA    R1,S99RB                build SVC99 parm list
         ST    R1,S99RBPTR             build SVC99 parm list
         OI    S99RBPTR,X'80'          build SVC99 parm list
         LA    R1,S99RBPTR
         DYNALLOC ,                    ALLOCATE THE DATA SET
         LTR   R15,R15
         BZ    ALLOC900                SVC99 went OK, jump
*
*        If SYSDSN ENQ failed, bypass ENQ and retry
*
ALLOC700 CLC   =X'0210',S99RB+4        SYSDSN ENQ failed?
         BNE   ALLOC900                no, jump
         BAL   R14,MODE_SUP         <- switch to MODE=SUP
         LTR   R13,R13                 Am I authorized?
         BNM   BROWSE                  no, quit
         OI    S99RB+16,X'20'          Do not enq on data set
         LA    R1,S99RBPTR
         DYNALLOC ,                    ALLOCATE THE DATA SET
         BAL   R14,MODE_PROB        <- switch back to MODE=PROB
         LTR   R15,R15                 Dynalloc OK this time?
         BNZ   BROWSE                  no, quit
         CLI   DSORG,DS1DSGPO          DSORG=PO?
         BE    OPENPO                  yes, jump
         MVI   USEBRIF,C'Y'            use BRIF, not BROWSE
*
*        allocation is successful
*
*
ALLOC900 CLC   DSORG,=AL2(DS1ORGAM)    DSORG=VSAM?
         BNE   OPENDA                  NO, JUMP
*----------------------------------------------------------------------
*                                                                     *
*        INITIALIZE VSAM DATA SET                                     *
*                                                                     *
*----------------------------------------------------------------------
OPENVS   MVC   ACBDDNM,DDNAME          MOVE DDNAME
         LA    R2,IFGACB               point to ACB
         ST    R2,OPENLIST             build open list
         MVI   OPENLIST,X'80'          build open list
         BAL   R14,MODE_SUP         <- switch to MODE=SUP
         OPEN  MF=(E,OPENLIST)         OPEN VSAM data set
         BAL   R14,MODE_PROB        <- switch back to MODE=PROB
         CH    R15,=H'4'               OPEN OK?
         BH    OPENVS7                 NO, QUIT
*
         LA    R3,MAXRECNO             point to work area
         SHOWCB ACB=(R2),AREA=(R3),LENGTH=12,                          X
               FIELDS=(NLOGR,LRECL,ENDRBA),MF=(G,BUFFER)
         ICM   R0,B'1111',MAXLRECL     LRECL=0 (LDS)
         BZ    OPENVS3                 yes, jump
*
         ST    R2,RPLDACB-IFGRPL+RPL1  STORE ACB ADDRESS INTO RPL
         ST    R2,RPLDACB-IFGRPL+RPL2  STORE ACB ADDRESS INTO RPL
         LA    R0,RECAD                THE RIGHT WORD
         ST    R0,RPLAREA-IFGRPL+RPL1  STORE AREA ADDRESS INTO RPL
         ST    R0,RPLAREA-IFGRPL+RPL2  STORE AREA ADDRESS INTO RPL
         LA    R0,L'RECAD              LENGTH
         ST    R0,RPLBUFL-IFGRPL+RPL1  STORE AREA LENGTH INTO RPL
         ST    R0,RPLBUFL-IFGRPL+RPL2  STORE AREA LENGTH INTO RPL
         LA    R0,LOWRBA               ARGUMENT FOR POINT
         ST    R0,RPLARG-IFGRPL+RPL2   STORE ARG ADDR INTO RPL
*
         TESTCB ACB=(R2),ATRB=RRDS,MF=(G,BUFFER)  is this an RRDS?
         BE    OPENVS4                 YES, JUMP
*
         ICM   R1,B'1111',MAXRECNO     MAX NUMBER OF RECORDS
         BNZ   OPENVS2                 NOT ZERO, JUMP
         L     R1,=F'1000000'          file open, use 1 million
*
OPENVS2  SLL   R1,2                    MULT BY 4
         LA    R0,0(,R1)               31-bit only
         L     R1,=F'1000000'          minimum value
         CLR   R0,R1                   more than 1 million ?
         BL    OPENVS2G                yes, jump
*
*        file has more than 256K rec, get between 1meg and nrec*4 bytes
*
         GETMAIN VRU,LV=((R0),(R1)),LOC=ANY,BNDRY=PAGE
         LR    R14,R0                  actual length gotten
         SRL   R14,2                   convert to RECNO
         ST    R14,MAXRECNO            remember MAXRECNO
         B     OPENVS2S
*
OPENVS2G GETMAIN RU,LV=(R0),LOC=ANY,BNDRY=PAGE
*
OPENVS2S STM   R0,R1,RBATABLE          STORE LENGTH, ORIGIN
         CLC   MAXLRECL,=F'32760'      IS RECSIZE TOO BIG?
         BNH   OPENVS2K                NO, JUMP
         MVC   MAXLRECL,=F'32760'      YES, TRUNCATE IT
*
*        allocate save area for record #1 (ESDS/KSDS)
*
OPENVS2K L     R0,MAXLRECL             record length
         GETMAIN RU,LV=(0),LOC=ANY     allocate save area for record 1
         STM   R0,R1,RECORD1           save length and address
*
         MVI   VS_TYPE,C'K'            Cluster is a KSDS
         TESTCB ACB=(R2),ATRB=KSDS,MF=(G,BUFFER)  is this a KSDS?
         BE    OPENVS6                 YES, JUMP
         MVI   VS_TYPE,C'E'            Cluster is an ESDS
         MVI   RPLOPT2-IFGRPL+RPL1,RPLADR    OPTCD=ADR
         B     OPENVS6
*
*        if LRECL=0, the data set is a linear data set (LDS).
*        use the screen width as record length
*
OPENVS3  MVI   VS_TYPE,C'L'            Cluster is an LDS
         CLOSE MF=(E,OPENLIST)         close the ACB, we don't need it
         LA    R0,L'MAXLRECL           length for VCOPY
         ST    R0,DWD                  length for VCOPY
         ISPLINK (=C'VCOPY ',=C'(ZSCREENW)',DWD,MAXLRECL,=C'MOVE ')
         PACK  DWD,MAXLRECL            convert to binary
         CVB   R0,DWD                  convert to binary
         ST    R0,MAXLRECL             screen width (80 or 132)
         DIV   IDENTIFY,               identify                        X
               ID=DIV_TOKN,            identifier                      X
               TYPE=DA,                data set                        X
               DDNAME=DDNAME-1,        X'08',C'ddname  '               X
               MF=(E,DIV_LIST)
         LTR   R15,R15                 OK?
         BNE   X'800'                  no, quit
         DIV   ACCESS,                 access                          X
               MODE=READ,              read-only                       X
               SIZE=MAXRECNO,          size in pages                   X
               MF=(E,DIV_LIST)
         CH    R15,=H'4'               OK?
         BH    X'804'                  no, quit
*
*        calculate the number of records (display lines)
*
         SLR   R0,R0
         L     R1,MAXRECNO             number of pages
         SLDL  R0,12                   number of bytes
         AL    R1,MAXLRECL             round up
         BCTR  R1,0                    round up
         D     R0,MAXLRECL             round up
         ST    R1,MAXRECNO             number of display lines
*
*        stage the first 16 pages
*
         LA    R0,BUFFER               DIV window
         ST    R0,CCW4                 pointer
         LA    R1,16                   SPAN
         ST    R1,LOWRBA               pointer
         XC    DIV_LIST(4),DIV_LIST    prevent S08B-0005
         DIV   MAP,                    access                          X
               OFFSET=0,               offset of 1st page              X
               SPAN=LOWRBA,            number of pages                 X
               AREA=CCW4,              pointer to my window            X
               MF=(E,DIV_LIST)
         LTR   R15,R15                 OK?
         BNE   X'808'                  no, quit
         B     OPENVS6
*
OPENVS4  MVI   VS_TYPE,C'R'            Cluster is an RRDS
*
OPENVS6  ISPLINK (=C'BRIF',DSNAME,=C'V ',MAXLRECL,=A(READVS00),0,DDATA)
         LR    R7,R15                  PASS RETURN CODE
         CLOSE MF=(E,OPENLIST)
         LM    R0,R1,RBATABLE          GET LENGTH, ORIGIN
         LTR   R0,R0
         BZ    SETMSG
         FREEMAIN RU,LV=(0),A=(1)      FREE TABLE OF POINTERS
         B     SETMSG
*
*        VSAM OPEN error
*
OPENVS7  CLI   ACBERFLG,160            empty data set?
         BNE   QUIT16                  no, quit
         ISPLINK (=C'SETMSG ',=C'ISRB603 ') "no data to browse"
         B     QUIT50                  free data set and exit
***********************************************************************
*                                                                     *
*        If DSORG=PO, invoke BROWSE with DATAID                       *
*                                                                     *
***********************************************************************
OPENPO   STRING 'LMINIT DATAID(BRDATAID) DDNAME(',DDNAME,') ENQ(SHRW)',X
               INTO=MESSAGE
         BAL   R14,ISPEXEC             invoke ISPEXEC service
         LTR   R15,R15                 went OK?
         BNZ   OPENDA                  no, jump
*
         LA    R0,L'BRDATAID           max length of dsname
         ST    R0,DWD                  max length of dsname
         ISPLINK (=C'VCOPY ',=C'(BRDATAID)',DWD,BRDATAID,=C'MOVE '),   X
               ERRET=OPENPO8
*
         STRING 'BROWSE DATAID(',BRDATAID,')',INTO=MESSAGE
         BAL   R14,ISPEXEC             invoke ISPEXEC service
         LR    R7,R15                  SAVE RETURN CODE FROM BROWSE
*
OPENPO8  STRING 'LMFREE DATAID(',BRDATAID,')',INTO=MESSAGE
         BAL   R14,ISPEXEC             invoke ISPEXEC service
         LTR   R15,R15                 went OK?
         BZ    SETMSG                  yes, exit
***********************************************************************
*                                                                     *
*        INITIALIZE BDAM DATA SET                                     *
*                                                                     *
***********************************************************************
OPENDA   LM    R14,R1,CAML28           CAMLST MODEL
         LA    R15,DSNAME              GET WORK AREA ADDRESS
         LA    R0,VOLSER               GET WORK AREA ADDRESS
         LA    R1,DS1FMTID             GET WORK AREA ADDRESS
         STM   R14,R1,TENWORDS         STORE UPDATED CAMLST
         OBTAIN TENWORDS               GET F1-DSCB
         LTR   R15,R15                 OK?
         BNZ   BROWSE                  NO, QUIT
         CLI   USEBRIF,C'Y'            Do we have to use BDAM?
         BE    OPENDA2                 YES, USE BDAM
         TM    DS1DSORG,DS1DSGDA       DSORG=DA?
         BO    OPENDA2                 YES, USE BDAM
         TM    DS1DSIND,DS1IND80       MULTI-VOLUME?
         BZ    OPENDA2                 YES, USE BDAM
         OC    DS1DSORG,DS1DSORG       DSORG=0?
         BZ    OPENDA2                 YES, USE BDAM
         OC    DS1BLKL,DS1BLKL         BLKSIZE=0?
         BZ    OPENDA2                 YES, USE BDAM
         CLI   DS1KEYL,0               KEYLEN>0
         BH    OPENDA2                 YES, USE BDAM
         TM    DS1RECFM,DS1RECFV+DS1RECFS   VS/VBS?
         BO    OPENDA2                 YES, USE BDAM
         OC    DS1LSTAR,DS1LSTAR       EOF=0 ?
         BNZ   BROWSE                  NO, USE STANDARD BROWSE
         TM    DS1SMSFG,DS1PDSE        is it a PDSE?
         BO    BROWSE                  YES, USE STANDARD BROWSE
*
*        determine the maximum lrecl
*
OPENDA2  DEVTYPE DDNAME,DVAREA,DEVTAB  get device type & track size
         LTR   R15,R15                 OK?
         BNZ   BROWSE                  NO, QUIT
         LH    R0,DS1LRECL             GET LRECL
         LTR   R0,R0
         BNZ   OPENDA2C
         ICM   R0,B'0011',DS1BLKL      GET BLKSIZE
         BNZ   OPENDA2C
         L     R0,DVAMAXRC             defaule blksize/lrecl
         STH   R0,DCBBLKSI             force BLKSIZE=32760 in DCB
*
OPENDA2C CL    R0,=F'32760'            record size too large?
         BNH   OPENDA2D                no, jump
         L     R0,=F'32760'            yes, truncate
*
OPENDA2D ST    R0,MAXLRECL             PASS IT TO BRIF
         LA    R2,DCB1                 DCB address
         MVC   DCBDDNAM,DDNAME         MOVE DDNAME
         ST    R2,OPENLIST             build open list
         MVI   OPENLIST,X'80'          build open list
*
OPENDA2E CLI   DS1DSORG,DS1DSGPO       DSORG=PO?
         BNE   OPENDA2O                no, jump
         MVI   DCBDSORG,DS1DSGPO       yes, prevent S013-14 in ESA
*
OPENDA2O OPEN  MF=(E,OPENLIST)         OPEN DCB1
         LTR   R15,R15                 OK?
         BNZ   BROWSE                  NO, QUIT
*
*        determine record format
*
OPENDA3  TM    DCBRECFM,DCBRECU        RECFM=U?
         BO    OPENDA3U                yes, jump
         MVI   RECFM,C'F'              set RECFM=F
         TM    DCBRECFM,DCBRECF        RECFM=F?
         BO    OPENDA4                 yes, jump
         MVI   RECFM,C'V'              set RECFM=V
         TM    DCBRECFM,DCBRECV        RECFM=V?
         BO    OPENDA4                 yes, jump
OPENDA3U MVI   RECFM,C'U'              RECFM=U or no RECFM
*
*        build IOB
*
OPENDA4  MVI   IOBFLAG1,IOBCMDCH+IOBSPSVC
         LA    R0,IOBNECB              ECB ADDRESS
         ST    R0,IOBECBPT             STORE ADDRESS
         LA    R0,CCW1                 FIRST CCW IN CHAIN
         ST    R0,IOBSTART             STORE ADDRESS
         ST    R2,IOBDCBPT             DCB ADDRESS
         LA    R15,SECTORNO            SECTOR NUMBER X'00'
         LA    R0,L'SECTORNO           LENGTH IS ONE BYTE
         STM   R15,R0,CCW1             SET SECTOR CCW
         MVI   CCW1+0,X'23'            COMMAND: SET SECTOR
         MVI   CCW1+4,CC               FLAGS: COMMAND CHAINING
*
         LA    R15,IOBCC               CCHHR ADDRESS
         LA    R0,5                    LENGTH IS FIVE BYTES
         STM   R15,R0,CCW2             SET SECTOR CCW
         MVI   CCW2+0,X'31'            COMMAND: SEARCH ID EQUAL
         MVI   CCW2+4,CC               FLAGS: COMMAND CHAINING
         LA    R15,CCW2                SEARCH CCW
         SLR   R0,R0                   2ND WORD
         STM   R15,R0,CCW3             TIC CCW
         MVI   CCW3+0,X'08'            COMMAND: TIC
*
         LA    R15,BUFFER              FIRST BLOCK
         SLR   R0,R0
         ICM   R0,B'0011',DVATRKLN     track length
         STM   R15,R0,CCW4             READ CCW
         MVI   CCW4,X'5E'              COMMAND CODE: READ MULTIPLE TRK
         OI    CCW4+4,SLI              FLAGS: SLI
*
*        Calculate the number of blocks per track
*
         TRKCALC FUNCTN=TRKCAP,                                        X
               TYPE=DVAUNIT,                                           X
               R=1,K=DCBKEYLE,DD=DCBBLKSI,                             X
               REGSAVE=YES,                                            X
               MF=(E,BUFFER)
         LTR   R15,R15
         BNZ   BROWSE                  TRKCALC ERROR
         ST    R0,BLKPTRK              NUMBER OF BLOCKS PER TRACK
         CLI   RECFM,C'F'              RECFM=F?
         BE    OPENDA8                 yes, jump
*
*        allocate RRNtable if RECFM=V or RECFM=U
*
OPENDA7  SLR   R0,R0
         ICM   R0,B'0111',DCBREL       number of tracks in data set
         SLL   R0,2                    mult by 4 (size of a word)
         GETMAIN RU,LV=(R0),LOC=ANY,BNDRY=PAGE
         STM   R0,R1,RBATABLE          STORE LENGTH, ORIGIN
         MVI   0(R1),X'7F'             end of table
         MVI   MAXRECNO,X'7F'          maximum record number
         B     OPENDA9
*
*        Calculate MAXRECNO for RECFM=F data sets
*
OPENDA8  SLR   R1,R1
         ICM   R1,B'0111',DCBREL       number of tracks in data set
         M     R0,BLKPTRK              number of blocks in data set
         LH    R14,DCBBLKSI            get blksize
         MR    R0,R14                  number of bytes  in data set
*PDUMP DCB1,ENDRBA+3
         D     R0,MAXLRECL             NUMBER OF RECORDS
         ST    R1,MAXRECNO             number of records in data set
*----------------------------------------------------------------------
*                                                                     *
*        Invoke BRIF                                                  *
*                                                                     *
*----------------------------------------------------------------------
OPENDA9  MVI   TTTR,X'FF'              force first read
         ISPLINK (=C'BRIF',DSNAME,=C'V ',MAXLRECL,=A(READDA00),0,DDATA)
         LR    R7,R15                  SAVE RETURN CODE FROM BRIF
***      TRACE 'CLOSDA'
         CLOSE MF=(E,OPENLIST)         CLOSE, FREE
         B     SETMSG
CAML28   CAMLST SEARCH,*-*,*-*,*-*
***********************************************************************
*                                                                     *
*        Invoke ISPF/PDF BROWSE                                       *
*                                                                     *
*        The BROWSE function is invoked in two situations:            *
*                                                                     *
*          1. the data set appears to be supported by BROWSE          *
*                                                                     *
*          or                                                         *
*                                                                     *
*          2. dynamic allocation failed, BROWSE will try to allocate  *
*             it also, will (probably) fail and issue an appropriate  *
*             message.                                                *
*                                                                     *
***********************************************************************
BROWSE   CLI   VOLSER,C' '             do we have a volser?
         BH    BROWSE2                 yes, jump
         ISPLINK (=C'BROWSE ',FULLDSN)
         LR    R7,R15                  save return-code from BROWSE
         B     SETMSG
BROWSE2  ISPLINK (=C'BROWSE ',FULLDSN,VOLSER)
         LR    R7,R15                  save return-code from BROWSE
***********************************************************************
*
*        PROGRAM EXIT
*
*           1. issue error message
*           2. clean up
*           3. goback to caller
*
***********************************************************************
SETMSG   LTR   R7,R7                   CHECK RETURN CODE FROM BRIF
         BZ    QUIT50                  OK, exit
SETMSG4  TRACE
         LA    R0,L'ZERRMSG
         ST    R0,DWD
         ISPLINK (=C'VCOPY ',=C'ZERRMSG ',DWD,ZERRMSG,=C'MOVE ')
         ISPLINK (=C'SETMSG ',ZERRMSG)
         B     QUIT50
*
QUIT16   LA    R7,16                   RC=16
*----------------------------------------------------------------------
*
*        free work area and goback to ISPF
*
*----------------------------------------------------------------------
QUIT50   TM    OPENLIST,X'80'          OPEN list OK?
         BNO   QUIT55                  no, jump
         CLOSE MF=(E,OPENLIST)         CLOSE ACB/DCB
*
         LM    R0,R1,RECORD1           first VSAM record
         LTR   R0,R0                   did we allocate it?
         BZ    QUIT55                  no, jump
         FREEMAIN RU,LV=(0),A=(1)      free record#1 save area
*
QUIT55   TM    S99RBPTR,X'80'          SVC 99 req blk OK?
         BNO   QUIT60                  no, jump
         MVI   S99RB+1,2               VERB=FREE
         OI    S99TUPTR,X'80'          ONLY 1 T.U.
         MVI   S99T1+1,1               X'55' -> X'01'
         LA    R1,S99RBPTR
         DYNALLOC ,                    FREE THE DATA SET
*
QUIT60   CLI   DIV_LIST,0              DIV in progress?
         BE    QUIT90                  no, jump
         XC    DIV_LIST(4),DIV_LIST    clear flags to prevent S08B-0005
         DIV   UNIDENTIFY,             remove access                   X
               MF=(E,DIV_LIST)
*
QUIT90   LR    R1,R13                  Work area
         L     R13,4(,R13)
         FREEMAIN R,LV=DYNAML,SP=1,A=(1)
         LR    R15,R7                  Return code
         RETURN (14,12),RC=(15)
***********************************************************************
*        Invoke ISPEXEC service
***********************************************************************
ISPEXEC  ST    R15,DWD                 LENGTH
         LA    R0,DWD                  LENGTH
         LA    R1,MESSAGE              text
         STM   R0,R1,TENWORDS          build parm list
         OI    TENWORDS+4,X'80'        build parm list
         L     R15,ISPEXEC@            addr of ISPEXEC
         LA    R1,TENWORDS             parm list
         BR    R15                     invoke ISPEXEC service
***********************************************************************
*        switch to MODE=SUP, if available
***********************************************************************
MODE_SUP L     R1,OWNCDE               point to my CDE
         TM    CDATTR2-CDENTRY(R1),CDSYSLIB   Am I from an auth. lib?
         BZR   R14                     no, goback
         LA    R15,11                  SDSF ESR#
         SLR   R0,R0                   OPTION CODE
         LA    R1,1                    OPTION CODE
ISFSVC   SVC   109                     TYPE-3 ESR
         LTR   R15,R15                 OK?
         BNZR  R14                     no, exit
         MODESET KEY=NZERO,MODE=SUP
         O     R13,=X'80000000'        I am running with MODE=SUP
         BR    R14                     goback
*
*        switch back to MODE=PROB
*
MODE_PROB LTR  R13,R13                 Am I in MODE=SUP?
         BPR   R14                     no, goback
         LA    R13,0(,R13)             reset MODE=SUP indicator
         STM   R14,R15,12(R13)         save R14-R15
         MODESET KEY=NZERO,MODE=PROB
         LM    R14,R15,12(R13)         restore R14-R15
         BR    R14                     goback
***********************************************************************
*                                                                     *
*        Scan data at the cursor position for a dsname                *
*                                                                     *
*        Some of this code has been lifted from ISPCDSN,              *
*        (file 270 of the CBT tape), I hope they don't mind.          *
*                                                                     *
*        The TLD address is retrieved from ISPTASK's register 1,      *
*        as done by ISPEXEC and ISPLINK.  If BR is invoked as a       *
*        program, ISPF passes the TLD address in the second word      *
*        of the parameter list built by the SELECT service; when      *
*        BR is invoked as a command, no TLD address is passed, as     *
*        far as I know.                                               *
*                                                                     *
***********************************************************************
CURSOR   L     R1,X'21C'               PSATOLD (MY TCB)
*LOOP
CURSOR11 L     R2,112(,R1)             TCBFSA
         CLI   40(R2),C'I'             R5 SLOT CONTAINS 'ISPF'
         BNE   CURSOR14                NO, JUMP
         CLI   24(R2),X'00'            IS R1 A 24-BIT ADDR?
         BNE   CURSOR14                NO, JUMP
         ICM   R2,B'1111',24(R2)       R1 SLOT (ADDR OF TLD PTR)
         BZ    CURSOR14                NO, JUMP
         CLI   0(R2),X'00'             IS IT A CLEAN 24-BIT ADDR?
         BNE   CURSOR14                NO, JUMP
         L     R2,0(,R2)               R2->TLD
         CLC   =C'TLD',0(R2)           IS THIS REALLY A TLD?
         BE    CURSOR19                YES, EXIT LOOP
CURSOR14 L     R1,132(,R1)             TCBOTC (MOTHER TASK)
         CL    R1,124(,R1)             IS IT THE JSTCB?
         BNE   CURSOR11                NO, CHECK FOR A TLD
*ENDLOOP
         B     CURSOR98                exit with "no-dsname" rc
*
*        TLD located, now get the line address and cursor
*        position.
*
CURSOR19 SLR   R0,R0
         LH    R1,166(,R2)             get cursor offset (TLDCSR)
         L     R3,192(,R2)             screen width (TLDCLSWD)
         DR    R0,R3                   get line # (R0) and offset (R1)
         LR    R5,R0                   cursor offset in line
         MR    R0,R3                   get offset to current line
         AL    R1,128(,R2)             change offset to addr (TLDxxxP)
         LR    R4,R1                   Current line in screen image
*----------------------------------------------------------------------
*
*        Scan current line at cursor position for a dsname
*
*              R4: current line
*              R5: cursor position
*
*----------------------------------------------------------------------
CURSOR21 MVI   TRNTBL,255
         MVC   TRNTBL+1(255),TRNTBL    init table
         SLR   R15,R15
         SLR   R0,R0
         BAL   R1,CURSOR22             branch around table, set R1
         DC    C'$',AL1(1)
         DC    C'.',AL1(1)
         DC    C'(',AL1(1)
         DC    C')',AL1(1)
         DC    C'#',AL1(2)             #@
         DC    C'a',AL1(9)
         DC    C'j',AL1(9)
         DC    C's',AL1(8)
         DC    C'A',AL1(9)
         DC    C'J',AL1(9)
         DC    C'S',AL1(8)
         DC    C'0',AL1(10)
         DC    X'00',0H'0'             end of table
*LOOP
CURSOR22 IC    R15,0(,R1)              FIRST BYTE
         IC    R0,1(,R1)               ITERATIONS
*--LOOP
CURSOR23 LA    R2,TRNTBL(R15)          point to TRNTBL+X'4A'
         MVI   0(R2),0                 STORE X'00' INTO TRNTBL+X'4A'
         LA    R15,1(,R15)             BUMP INDEX
         BCT   R0,CURSOR23             NEXT CHARACTER
*--ENDLOOP
         LA    R1,2(,R1)               NEXT ENTRY IN TABLE
         CLI   0(R1),0                 END OF TABLE?
         BNE   CURSOR22                NEXT STRING
*ENDLOOP
*
*        scan current line at cursor position to extract the dsn
*
         LA    R3,0(R5,R4)             R3-> cursor location
         CLI   0(R3),C'('              Is it left paren?
         BZ    CURSOR71                Yes, extract within parens
         TRT   0(1,R3),TRNTBL          Are we on data set name?
         BZ    CURSOR60                Yes, begin scanning for start
         CLI   0(R3),C''''             Is it a quote?
         BNZ   CURSOR98                No, cursor invalid
         MVI   QFLAG,X'80'             Yes, indicate left quote
         TRT   1(1,R3),TRNTBL          Is dsn to right of quote?
         BZ    CURSOR80                Yes, prepare to extract it
         LTR   R5,R5                   Cursor in pos 1?
         BZ    CURSOR98                yes, this is bad
         BCTR  R3,0                    No,  check left of quote
         TRT   0(1,R3),TRNTBL          Was it there?
         BNZ   CURSOR98                No,  cursor invalid
         MVI   QFLAG,X'01'             Yes, indicate right quote
*
*        scan backwards to locate beginning of dsname
*
CURSOR60 LTR   R5,R5                   Cursor in pos 1?
         BZ    CURSOR81                yes, dsn starts in pos 1
         LR    R0,R3                   current position
         SR    R0,R4                   subtract addr of pos 1
         BNP   CURSOR98                bad, quit
*loop
CURSOR62 BCTR  R3,0                    R3-> cursor location
         TRT   0(1,R3),TRNTBL          Byte in table?
         BNZ   CURSOR70                No,  found left delimiter
         BCT   R0,CURSOR62             Yes, decr offset; check next
*endloop
         B     CURSOR81                dsname starts in pos 1
*
*        R3 now points to start of dsname
*
CURSOR70 CLI   0(R3),C'('              Was delimiter left paren?
         BNE   CURSOR75                No, include parens in dsn
CURSOR71 MVI   TRNTBL+C')',C')'        Make right paren a delimiter
         MVI   TRNTBL+C'(',0           Include parens in dsn if present
         B     CURSOR80                Extract withing parens
CURSOR75 MVI   TRNTBL+C'(',0           Include parens in dsn if present
         CLI   0(R3),C''''             Was delimiter a quote?
         BNE   CURSOR80                No,  leave bits unchanged
         OI    QFLAG,X'80'             Yes, indicate left quote
*
*        scan dsname to locate its right boundary
*
CURSOR80 LA    R3,1(,R3)               R3-> left position of dsname
*
CURSOR81 TRT   0(56,R3),TRNTBL         Scan until not in table
         BZ    CURSOR98                end dlm not found, exit
         LR    R2,R1                   R2-> right delimiter
         CLI   0(R2),C''''             Was delimiter a quote?
         BNE   CURSOR90                No,  leave bits unchanged
         OI    QFLAG,X'01'             Yes, indicate right quote
*
CURSOR90 TM    QFLAG,X'81'             Quotes surrounding dsname ?
         BM    CURSOR98                no matching quotes
         LR    R0,R2                   R0 = first byte after dsname
         LR    R1,R3                   R1 = first byte of dsname
         SLR   R0,R1                   R0 = length
         BR    R14                     exit
*
CURSOR98 SLR   R0,R0                   no dsname
         SLR   R1,R1                   no dsname
*
CURSOR99 BR    R14                     GOBACK
***********************************************************************
*
*        VSAM READ ROUTINE
*
***********************************************************************
$FARRTNE LOCTR
READVS00 SAVE  (14,12),,'XBRVS - VSAM READ RTNE'
         LM    R4,R7,0(R1)             INPUT PARMS
         L     R7,0(,R7)               A(SAVERRTN)
         ST    R13,4(,R7)
         ST    R7,8(,R13)
         LR    R13,R7
         USING SAVERRTN,R13
         LM    R11,R12,BASEREGS
*
READVS10 TRACE
         L     R3,0(,R6)               RECORD NUMBER (FROM BROWSE)
         C     R3,MAXRECNO             BEYOND END OF FILE?
         BH    READVS80                YES, EXIT
*
         LA    R7,RPL1                 OPTCD=SEQ
         USING IFGRPL,R7
         CLI   VS_TYPE,C'R'            is this an RRDS?
         BE    RRDS00                  YES, JUMP
         CLI   VS_TYPE,C'L'            is this an LDS?
         BE    LDS000                  YES, JUMP
*
*        check rbatable if record has already been read
*
READVS30 LR    R2,R3                   RRN
         BCTR  R2,0                    RELATIVE TO ZERO
         SLL   R2,2                    MAKE IT AN INDEX IN RBA TABLE
         AL    R2,RBATABLE+4           CHANGE OFFSET TO ADDRESS
         ICM   R2,B'1111',0(R2)        GET RBA OF RECORD
         BNZ   READVS50                ALREADY READ, JUMP
*
*        the record requested has never been read; read the file
*        sequentially until the desired record is reached.
*
         L     R8,HIGH_RRN             LAST RECORD READ SEQUENTIALLY
*LOOP
READVS31 TRACE
         GET   RPL=RPL1                GET NEXT RECORD
         LTR   R15,R15                 OK?
         BZ    READVS45                YES, JUMP
*
*        actual end of data (should only occur when SHOWCB
*        returned MAXRECNO=0)
*
         CLI   RPLFDB3,X'04'           end of data?
         BE    READVS81                YES, exit
*
*        The first time a spanned record is read (FDBK=X'84'),
*        switch RPL1 and RPL2 from OPTCD=LOC to OPTCD=MVE.
*        From that point on, all GETs will be move mode.
*        Note that a cluster defined with the SPANNED attribute
*        does not necessarily contain spanned records (ie records
*        made up of more than one segment).
*
         CLI   RPLFDB3,X'48'           OPTCD=KEY with DATA component?
         BE    READVS39                YES, JUMP
*
         CLI   RPLFDB3,X'84'           OPTCD=LOC WITH SPANNED RECORDS?
         BNE   READVS90                no, error
*
         NI    RPLOPT1,255-RPLLOC      OPTCD=MVE
         NI    RPLOPT1-IFGRPL+RPL2,255-RPLLOC  OPTCD=MVE
         LA    R0,BUFFER               THE RIGHT WORD
         ST    R0,RPLAREA              STORE AREA ADDRESS INTO RPL
         ST    R0,RPLAREA-IFGRPL+RPL2  STORE AREA ADDRESS INTO RPL2
         L     R0,=F'65536'            AREALEN=65536
         ST    R0,RPLBUFL              AREALEN=65536
         ST    R0,RPLBUFL-IFGRPL+RPL2  AREALEN=65536 INTO RPL2
         B     READVS40
*
READVS39 MVI   VS_TYPE,C'E'            Cluster is an ESDS
         MVI   RPLOPT2-IFGRPL+RPL1,RPLADR    OPTCD=ADR
*
*        re-read a record after X'48' or X'84'
*
READVS40 TRACE
         GET   RPL=RPL1                RE-READ RECORD AFTER 48 OR 84
         LTR   R15,R15                 OK?
         BNZ   READVS90                NO, QUIT
*
*        move first record to its save area
*
READVS45 CH    R3,=H'1'                is this record #1 ?
         BNZ   READVS46                no, jump
         LM    R15,R0,RECORD1          length,address of save area
         LR    R14,R0                  "TO" address
         L     R0,RECAD                "FROM" address
         L     R1,RPLRLEN              "FROM" length
         MVCL  R14,R0                  move record #1 to its save area
         MVC   RECORD1+8(4),RPLRLEN    save actual length
*
*        store RBA into RBATABLE for the next time this record is read
*
READVS46 LR    R2,R8                   RRN
         SLL   R2,2                    MAKE IT AN INDEX IN RBA TABLE
         AL    R2,RBATABLE+4           CHANGE OFFSET TO ADDRESS
         MVC   0(4,R2),RPLDDDD         SAVE RBA OF RECORD
         LA    R8,1(,R8)               BUMP CURRENT RRN
         CR    R8,R3                   Was it the record needed?
         BL    READVS31                no, get next
*ENDLOOP
         ST    R3,HIGH_RRN             SAVE RRN OF RECORD
         L     R1,RBATABLE+4           point to rbatable
         MVI   0(R1),255               dummy RBA for record 1
         B     READVS70
*
*        re-read a record already read, using the RBA kept in the table
*
READVS50 TRACE
         ST    R2,LOWRBA               store RBA into ARG
         LA    R7,RPL2                 OPTCD=DIR
         LA    R0,LOWRBA               specify search argument
         ST    R0,RPLARG               store ARG addr into RPL2
         NI    RPLOPT1,255-RPLSEQ      clean up OPTCD=SEQ
         OI    RPLOPT1,RPLDIR          set OPTCD=(ADR,DIR,KGE)
         MVI   RPLOPT2,RPLADR          set OPTCD=(ADR,DIR,KGE)
         GET   RPL=RPL2                re-read record
         LTR   R15,R15                 OK?
         BZ    READVS70                yes, jump
*
*        re-read record #1
*
*        This sequence works around the fact that you get
*        R15=8 and FDB3=X'20' if you try to read the first
*        record with RBA=00000000.  The first record has been
*        saved into its own save area the first time it was read.
*
         CLI   RPLFDB3,X'20'           invalid RBA?
         BNE   READVS59                no, jump
         CH    R3,=H'1'                was this the first record?
         BNE   READVS59                no, jump
         LM    R1,R2,RECORD1+4         address,length of record 1
         ST    R2,RPLRLEN              length of record 1
         B     READVS74                pass record #1
*
READVS59 STRING '** ERROR ** RPLFDBWD=',(RPLFDBWD,,X),                 X
               ' REQ=',(RPLREQ,,X),' OPTCD=',(RPLOPTCD,2,X),           X
               ' RRN=',((R3),,L),' RBA=',(LOWRBA,,X),                  X
               INTO=MESSAGE
         SLR   R15,R15                 RC=00
         B     READVS95                ISSUE MESSAGE
*
*        KEYED ACCESS (RRDS)
*
RRDS00   ST    R3,LOWRBA               RRN
         LA    R7,RPL2                 OPTCD=DIR
         MVI   RPLOPT1,RPLLOC+RPLDIR   OPTCD=(DIR,KEQ,LOC,KEY)
         MVI   RPLOPT2,RPLKEY          OPTCD=(DIR,KEQ,LOC,KEY)
         GET   RPL=RPL2                GET RECORD (RRDS)
         LTR   R15,R15                 OK?
         BZ    READVS70                YES, JUMP
         STRING '** ERROR ** RPLFDBWD=',(RPLFDBWD,,X),              XXXX
               ' RRN=',((R3),,L),INTO=MESSAGE
         SLR   R15,R15                 RC=00
         B     READVS95                ISSUE MESSAGE
*
*        return record just read to BRIF
*
READVS70 L     R1,RPLAREA              NO, GET RECORD ADDR (OPTCD=MVE)
         TM    RPLOPTCD,RPLLOC         OPTCD=MVE?
         BZ    READVS74                YES, JUMP
         L     R1,0(,R1)               GET RECORD ADDR (OPTCD=LOC)
READVS74 ST    R1,0(,R4)               PASS RECORD ADDRESS TO BRIF
         MVC   0(4,R5),RPLRLEN         PASS RECORD LENGTH TO BRIF
         L     R13,4(,R13)
         RETURN (14,12),RC=00
*
*        Linear data set
*
LDS000   LR    R2,R3                   RRN
         BCTR  R2,0                    RELATIVE TO ZERO
         MH    R2,MAXLRECL+2           make it an offset
         LA    R2,BUFFER(R2)           CHANGE OFFSET TO ADDRESS
         ST    R2,0(,R4)               return record address to BRIF
         MVC   0(4,R5),MAXLRECL        PASS RECORD LENGTH TO BRIF
         L     R13,4(,R13)
         RETURN (14,12),RC=00
*
*        record number too high, return number of highest rcd to BRIF
*
READVS80 MVC   0(4,R6),MAXRECNO        RETURN MAX RECORD NUMBER
READVS81 L     R13,4(,R13)
         RETURN (14,12),RC=8
*
*        error in GET macro, show feedback code
*
READVS90 STRING '** ERROR ** RPLFDBWD=',(RPLFDBWD,,X),              XXXX
               ' RRN=',((R3),,L),' HIGH_RRN=',((R8),,L),             XXX
               INTO=MESSAGE
         ST    R3,0(,R6)               END OF FILE
         LA    R15,008                 RC=8 (EOF)
.@TEST2 SR 15,15
*
*        return error message to BRIF
*
READVS95 LA    R0,MESSAGE              MESSAGE
         ST    R0,0(,R4)               PASS ADDRESS
         LA    R0,L'MESSAGE            LENGTH
         ST    R0,0(,R5)               PASS LENGTH
         L     R13,4(,R13)
         RETURN (14,12),RC=(15)
***********************************************************************
*
*        NON-VSAM READ ROUTINE
*
*        This routine reads records from a non-vsam data-set
*        by issuing EXCP macros against a BDAM DCB.
*
*        This method (EXCP+BDAM DCB) offers the following advantages:
*
*        a. the program has access to all of the volumes of a
*           multi-volume data set at the same time, with a single
*           OPEN, a single DCB, and no need to go through EOV
*
*        b. full tracks may be read, which speeds up access
*           to un-blocked or poorly-blocked data sets
*
***********************************************************************
READDA00 SAVE  (14,12),,'XBRVS - BDAM READ RTNE'
         LM    R4,R7,0(R1)             parameters from BRIF
         STM   R4,R7,TENWORDS          save them
         L     R7,0(,R7)               A(SAVERRTN)
         ST    R13,4(,R7)
         ST    R7,8(,R13)
         LR    R13,R7
         USING SAVERRTN,R13
         LM    R11,R12,BASEREGS
         L     R3,0(,R6)               RECORD NUMBER (FROM BROWSE)
         CL    R3,MAXRECNO             beyond end of file?
         BH    READDA80                yes, exit
         CLI   RECFM,C'F'              RECFM=F?
         BE    READDA30                yes, jump
*----------------------------------------------------------------------
*
*        Process RECFM=V/VB/U records
*
*        This routine reads the data set sequentially and
*        builds a table of high RRN (relative record number)
*        for each track.  This table is used to remember which
*        track holds a particular record.
*
*----------------------------------------------------------------------
READDA20 L     R1,RBATABLE+4           point to table
*PDUMP DCB1,ENDRBA+3
         BAL   R15,*+8                 set R15, skip LA first time
*--loop
         LA    R1,4(,R1)               bump index
         CL    R3,0(,R1)               record on this track?
         BHR   R15                     no, try next one
*--endloop
*PDUMP BUFFER,BUFFER+256
         LR    R3,R1                   address of table entry
         SH    R1,=H'4'                previous entry
         ST    R1,ENDRBA               save address of prev table entry
*
         SL    R3,RBATABLE+4           offset
         SRL   R3,2                    track number
         SLL   R3,8                    TTT0
         BAL   R14,EXCP00          <-- read a track
         LTR   R15,R15                 EOF?
         BZ    READDA21                no, jump
*
*        out-of-extents condition
*
         L     R1,ENDRBA               load address of prev table entry
         MVC   MAXRECNO,0(R1)          last RRN on last track
         B     READDA80                exit with RC=8
*CLC BUFFER(4),IOBSEEK+3
*BE XXX
*L R2,0(,R6)
*PDUMP DCB1,ENDRBA+3
*XX EQU *
*
*        scan track buffer to count records
*
READDA21 L     R1,ENDRBA               load address of table entry
         CLI   4(R1),X'7F'             end-of-table flag?
         BNE   READDA24                no, skip buffer scan
         SLR   R1,R1                   start of track
         SLR   R2,R2                   record number on this track
*--loop
READDA22 BAL   R14,GETREC              scan buffer
         LA    R2,1(,R2)               count records on this track
         LTR   R1,R1                   end of buffer reached?
         BNZ   READDA22                no, get next record
*--endloop
         BCTR  R2,0                    ignore last call
         STC   R2,TTTR+3               save number of records
*
*        store current track's RRN into rbatable
*
         L     R1,ENDRBA               load address of prev table entry
         CL    R1,RBATABLE+4           first track?
         BL    READDA23                yes, jump
         AL    R2,0(,R1)               no, add previous track's RRN
READDA23 ST    R2,4(,R1)               fill in table entry
         MVI   8(R1),X'7F'             set table stop
         L     R3,0(,R6)               RRN from BRIF
         B     READDA20                keep on reading sequentially
*endloop
*L R3,0(,R6)
*L R1,RBATABLE+4
*LA 0,00(1)
*LA 1,63(1)
*PDUMP (R0),(R1)
*
READDA24 L     R3,0(,R6)               RRN from BRIF
         L     R1,ENDRBA               load address of prev table entry
         CL    R1,RBATABLE+4           first track?
         BL    READDA25                yes, jump
         SL    R3,0(,R1)               no, subtract RRN of prev trk
*
READDA25 SLR   R1,R1                   start of track
*loop
READDA26 BAL   R14,GETREC              scan buffer
         BCT   R3,READDA26             until my record is reached
*endloop
*****    LA    R1,8(,R1)               data address
         B     READDA70                goback
*----------------------------------------------------------------------
*
*        process RECFM=F/FB records
*
*        This routine assumes that the data set is RECFM=FS or FBS,
*        i.e. that it does not contain truncated records.
*        This assumption allows the track address of a particular
*        record to be calculated without reading the preceding
*        records.  For example, to access record # 1111 in a
*        data set that has 10 blocks per track and 10 records per
*        block, track 11 is read, then record # 1 of the block # 2
*        is returned to BRIF (1111 = 11*100 + 10 + 1)
*
*----------------------------------------------------------------------
READDA30 BCTR  R3,0                    first record is R0
         M     R2,MAXLRECL             OFFSET IN BYTES
         LH    R14,DCBBLKSI            PICK UP BLKSIZE
         DR    R2,R14                  BLOCK NUMBER IN R3, OFFSET IN R2
         ST    R2,0(,R4)               save offset
*
*        R3 = relative block address (first block is zero)
*
         SLR   R2,R2
         D     R2,BLKPTRK              get relative track number
         LA    R2,1(,R2)               first record on track is R1
         SLL   R3,8                    R3 = TTTTTT00
         OR    R3,R2                   R3 = TTTTTTRR
         BAL   R14,EXCP00          <-- read a track
         LTR   R15,R15
         BNZ   READDA80                out-of-extents
*
*        scan the buffer for the right block (R3 contains record #)
*
         SLR   R1,R1                   start of track
*loop
READDA32 BAL   R14,GETREC              scan buffer
         LTR   R1,R1                   end of buffer reached?
         BZ    READDA55                yes, error
         CLM   R3,B'0001',4(R1)        is this my record number?
         BNE   READDA32                no, try next block
*endloop
         ICM   R0,B'0111',5(R1)        is this an EOF?
         BZ    READDA51                yes, exit
         L     R0,MAXLRECL             maximum record length
         LA    R1,8(,R1)               point to the first record
         AL    R1,0(,R4)               add record offset
         B     READDA70                return record to BRIF
*
READDA51 L     R0,0(,R4)               load record offset
         LTR   R0,R0                   first record?
         BNZ   READDA55                no, jump
         LA    R0,L'READDA52           message length
         BAL   R1,READDA70             branch around message
READDA52 DC    C'**EOF**',0H'0'
*
READDA55 LA    R0,L'READDA56           message length
         BAL   R1,READDA70             branch around message
READDA56 DC    C'**NO RECORD FOUND**',0H'0'
*
READDA70 ST    R1,0(,R4)               PASS RECORD ADDRESS TO BRIF
READDA71 ST    R0,0(,R5)               PASS RECORD LENGTH TO BRIF
         L     R13,4(,R13)
         RETURN (14,12),RC=00
*
*        return to BRIF with RC=8 (high RRN)
*
READDA80 MVC   0(4,R6),MAXRECNO        RETURN MAX RECORD NUMBER
         L     R13,4(,R13)
         RETURN (14,12),RC=8        <- RC=8   (return highest rrn)
*----------------------------------------------------------------------
*        read a track
*----------------------------------------------------------------------
EXCP00   STM   R2,R14,12(R13)
         CLM   R3,B'1110',TTTR         track in buffer already?
         BE    EXCP80                  YES, JUMP
         ST    R3,TTTR                 STORE TRACK ADDRESS
*
*        convert TTRN to IOBSEEK (MBBCCHHR)
*
         L     R0,TTTR                 load TTTR
         SRL   R0,8                    make it 0TTT
         SLL   R0,16                   make it TT00
         L     R1,DCBDEBAD             DCBDEBAD
         LA    R2,IOBSEEK              MBBCCHHR FIELD
         LR    R6,R13                  SAVE R13
         L     R15,16                  CVTPTR
         L     R15,28(,R15)            CVTPCNVT
         BALR  R14,R15                 CONVERT TTRN TO MBBCCHHR
         LM    R6,R13,28(R6)           RESTORE R6-R13
         LTR   R15,R15                 OK?
         BNZ   EXCP90                  no, eof
         EXCP  IOBSTDRD             <- read a track
         WAIT  1,ECB=IOBNECB           wait for completion
*PDUMP BUFFER,BUFFER+400
EXCP80   SLR   R15,R15                 RC=00
         CLC   BUFFER(4),IOBSEEK+3     CHECK CCHH
         BE    EXCP90                  OK, exit with r15=0
         LA    R15,4                   no, eof
*
EXCP90   LM    R2,R14,12(R13)          restore registers
         BR    R14
*----------------------------------------------------------------------
*
*        scan read buffer
*
*----------------------------------------------------------------------
GETREC   CLI   RECFM,C'V'              RECFM=V?
         BE    GETRECV1                yes, jump
*
*        RECFM=U
*
         LTR   R1,R1                   First time?
         BZ    GETREC8                 yes, jump
         SLR   R15,R15
         ICM   R15,B'0011',6(R1)       get record length
         SLR   R0,R0
         IC    R0,5(,R1)               get key length
         LA    R1,8(R15,R1)            l'count+l'data
         ALR   R1,R0                   add l'key
         CLC   0(4,R1),IOBSEEK+3       is this my CCHH?
         BE    GETREC9                 yes, process it
         SLR   R1,R1                   no, end-of-track
         BR    R14
GETREC8  LA    R1,BUFFER               start-of-track
GETREC9  SLR   R0,R0
         ICM   R0,B'0011',6(R1)        block size (from count+6)
         CL    R0,MAXLRECL             too big?
         BNHR  R14                     no, goback
         L     R0,MAXLRECL             yes, truncate
         BR    R14
*
*        RECFM=V
*
GETRECV1 LTR   R1,R1                   First time?
         BZ    GETRECV5                yes, jump
         CLC   MAXLRECL+2(2),0(R1)     RDW length OK?              @133
         BL    GETRECV2                no, EOF                     @133
         AH    R1,0(,R1)               add length from RDW
         CL    R1,RECAD                end of block?
         BL    GETRECV9                not yet, process this record
         CLC   0(4,R1),IOBSEEK+3       is this my CCHH?
         BE    GETRECV8                yes, process it
GETRECV2 SLR   R1,R1                   no, end-of-track
         BR    R14
GETRECV5 SLR   R0,R0                                               @133
         SLR   R1,R1                                               @133
         ICM   R0,B'1100',BUFFER+6     get DD from CCHHRKDD        @133
         CL    R0,BUFFER+8             is this a BDW?              @133
         BNER  R14                     no, EOF                     @133
         LA    R1,BUFFER               start-of-track
GETRECV8 SLR   R15,R15
         ICM   R15,B'0011',6(R1)       block size (from count+6)
         LA    R0,7(R15,R1)            last byte of current block
         ST    R0,RECAD                save end of block address
         LA    R1,8(,R1)               skip count, point to BDW
         LA    R1,4(,R1)               first record in this block
GETRECV9 LH    R0,0(,R1)               length of the record from RDW
         BR    R14
*----------------------------------------------------------------------
*        MODEL DCB, ACB, RPL
*----------------------------------------------------------------------
*        PRINT NOGEN
ACB1_M ACB     MACRF=(ADR,KEY,SEQ,DIR,IN),STRNO=2,RMODE31=ALL,       XXX
               BUFSP=200000
RPL1_M   RPL   ACB=*-*,OPTCD=(KEY,SEQ,LOC,NSP),AREA=*-*,AREALEN=4
RPL2_M   RPL   ACB=*-*,OPTCD=(ADR,DIR,LOC,KGE),AREA=0,AREALEN=4,ARG=0
DCB1_M   DCB   DSORG=DA,MACRF=RIC,DDNAME=SYSUT1
MODELS_L EQU   *-ACB1_M
         STRING FINAL_CALL
*----------------------------------------------------------------------
*        DEFINE INPUT PARAMETERS FOR IKJPARS                          *
*----------------------------------------------------------------------
PCLCSECT IKJPARM
DSNPCE   IKJPOSIT DSNAME,LIST,USID,PROMPT='DATA SET NAME',             X
               HELP='NAME OF THE DATA SET TO BE BROWSED'
VOLPCE   IKJKEYWD
         IKJNAME 'VOLUME',SUBFLD=VOLSUBF,ALIAS='VOLSER'
VOLSUBF  IKJSUBF
VOLPCE2  IKJIDENT 'VOLUME',MAXLNTH=6,FIRST=ALPHANUM,OTHER=ALPHANUM
         IKJENDP
***********************************************************************
*        Dynamic Storage           <-- R13
***********************************************************************
DYNAM    DSECT
         DS    18F                     SAVE AREA (MAIN LINE)
SAVERRTN DS    18F                     SAVE AREA (READ ROUTINE)
BASEREGS DS    A(XBRVS,XBRVS+4095)
TENWORDS DS    10F                     LEN/ADDR OF BDAM BUFFER
DWD      DS    D                       VCOPY
DIV_TOKN DS    D                       DIV token
DIV_LIST DIV   IDENTIFY,MF=L           DIV parm list
ZAPPLID  DS    C'ISR     '             current ISPF application ID
ZDLDSORG DS    C'VS      '             data set org from PDF 3.4 panel
ZERRMSG  DS    C'12345678'             ERROR MESSAGE
BRDATAID DS    C'ISR12345'             DATA ID from LMINIT
DDATA    DS    A(SAVERRTN)             BASE ADDRESS
ISPLINK@ DS    V(ISPLINK)              ISPF interface
ISPEXEC@ DS    V(ISPEXEC)              ISPF interface
RBATABLE DS    A(0,0)                  RBA TABLE
HIGH_RRN DS    F                       RRN OF LAST RECORD
DYNANSWR DS    F                       LEN/ADDR OF BDAM BUFFER
ECB1     DS    F                       LEN/ADDR OF BDAM BUFFER
OWNCDE   DS    A(CDENTRY)              my own CDE
OWNNAME  DS    CL8                     my true name
VOLGIVEN DS    C'Y'                    VOLSER given by user
USEBRIF  DS    C'Y'                    use BRIF, not standard BROWSE
VS_TYPE  DS    C'K'                    VSAM type: K E R
RECFM    DS    C'F'                    RECFM: F/V/U
OPENLIST OPEN  IFGACB,MF=L
         IFGACB DSECT=NO
RPL1     RPL   ACB=IFGACB,OPTCD=(KEY,SEQ,LOC)
RPL2     RPL   ACB=IFGACB,OPTCD=(ADR,DIR,LOC)
DCB1     DCB   DSORG=DA,MACRF=RIC,OPTCD=R,DDNAME=SYSUT1
DCBKEYLE EQU   DCB1+16,1,C'F'          key length
DCBREL   EQU   DCB1+17,3,C'F'          number of tracks in data set
DCBDSORG EQU   DCB1+26,2,C'X'          data set org
DCBDDNAM EQU   DCB1+40,8,C'C'          ddname
DCBDEBAD EQU   DCB1+44,4,C'A'          DEB pointer
DCBBLKSI EQU   DCB1+62,2,C'H'          BLKSIZE
DCBRECFM EQU   DCB1+36,1,C'X'          RECFM
DCBRECU  EQU   X'C0'
DCBRECF  EQU   X'80'
DCBRECV  EQU   X'40'
*
         IEZIOB DSECT=NO
CCW1     CCW   X'23',SECTORNO,CC,1     SET SECTOR
CCW2     CCW   X'31',IOBCC,CC,5        SEARCH ID EQUAL
CCW3     CCW   X'08',CCW2,0,1          TIC
CCW4     CCW   X'1E',BUFFER,SLI,8+256  READ KEY+DATA
CC       EQU   X'40'                   COMMAND CHAINING
SLI      EQU   X'20'                   SUPPRESS LENGTH ERROR
         IHADVA DSECT=NO               work area for DEVTYPE macro     X
                                       must be aligned on a FWD bdy
SECTORNO DS    FL1'0'                  Sector number for RPS devices
*
RECAD    DS    A                       BLOCK NUMBER
LOWRBA   DS    F                       used for POINT and RRDS GET
TTTR     DS    F                       USED FOR BDAM
BLKPTRK  DS    F                       Number of blocks per track
RECORD1  DS    F,A,F                   Length,address of 1st VS record
MAXRECNO DS    F'12345678'          +0 MAX RECORD NUMBER
MAXLRECL DS    F'32760'             +4 MAX LRECL
ENDRBA   DS    F'32760'             +8 ENDRBA FOR RRDS
*------- BLOCS POUR ALLOCATION DYNAMIQUE DU FICHIER -------------------
S99RBPTR DS    A(S99RB+X'80000000')
S99RB    DS    A(X'14012000',0,S99TUPTR,0,0)
S99TUPTR DS    10A
S99T1    DS    X'0055,0001',AL2(L'DDNAME)   RETURN DDNAME
DDNAME   DS    CL8       '
S99T2    DS    X'0002,0001',AL2(L'DSNAME)   DSNAME
DSNAME   DS    CL44,C                       data set name
S99T3    DS    X'0003,0001',AL2(L'MEMBER)   MEMBER
MEMBER   DS    CL8                          member name
S99T57   DS    X'0057,0001',AL2(L'DSORG)    RETURN DSORG
DSORG    DS    X'0000'
S99T5D   DS    X'005D,0001',AL2(L'VOLSER)   RETURN VOLSER
VOLSER   DS    C'VOLSER'
*----------------------------------------------------------------------
BUFFER   DS    0D                      SPANNED RECORDS
         IECSDSL1 1                    F1-DSCB
QFLAG    DS    5X                      CCHHR
         AIF   (D'DS1SMSFG).SMS4
DS1SMSFG EQU   IECSDSL1+X'4A',1,C'X'   SMS flags
.SMS4    AIF   (D'DS1PDSE).SMS5
DS1PDSE  EQU   X'08'                   data set is a PDSE
.SMS5    ANOP
FULLDSN  DS    CL56                    'data.set(member)'
MESSAGE  DS    CL256
TRNTBL   DS    CL256                   dsname scan
CLEARLEN EQU   *-DYNAM
         ORG   BUFFER+65536            buffer is 16 pages
DYNAML   EQU   *-DYNAM
*----------------------------------------------------------------------
         IKJCPPL                       Command Buffer Parm List
         IKJPPL                        Parse Parm List (IKJPARS)
         IKJECT                        Environment Control Table
         YREGS
IFGRPLVS OPSYN ANOP
         IFGRPL DSECT=YES
         IHACDE ,                      Contents Directory Entry
         END
//SYSLIB   DD DISP=SHR,DSN=SYS1.MACLIB
//         DD DISP=SHR,DSN=SYS1.MODGEN
//SYSUT1   DD UNIT=SYSDA,SPACE=(CYL,2)
//SYSPRINT DD SYSOUT=*
//SYSLIN   DD DISP=(,PASS),UNIT=SYSDA,SPACE=(TRK,1),DCB=BLKSIZE=3120
//*-------------------------------------------------------------------
//LKED     EXEC PGM=HEWL,PARM='MAP,RENT'
//SYSLIN   DD DSN=*.ASMH.SYSLIN,DISP=(OLD,PASS)
//         DD *
 ALIAS BR
 NAME ISF$BRO(R)
//SYSLMOD  DD DISP=SHR,DSN=TCNAMS1.LOAD
//SYSPRINT DD SYSOUT=*
//
  //TCNAMS1O JOB (ACCT#),OFFLOAD,
  // NOTIFY=TCNAMS1,
  // CLASS=A,MSGCLASS=X,COND=(4,LT)
  //*
  //*    CREATE A MULTI-VOLUME DATA SET
  //*
  //ASMH EXEC PGM=IEV90,PARM=(OBJECT,NODECK,NOESD,NORLD,NOXREF)
          $ENTER CTR=K1
  PUN5    $PUNCH K1,CTR=K1
           B     PUN5
          $SUBS
           END
  //SYSPRINT DD SYSOUT=*
  //SYSLIB DD DISP=SHR,DSN=TCNAMS1.MACLIB
  //SYSUT1 DD UNIT=SYSDA,SPACE=(CYL,1)
  //SYSLIN DD UNIT=SYSDA,SPACE=(TRK,1),DISP=(,PASS),DCB=BLKSIZE=3200
  //GO EXEC PGM=LOADER,PARM=NOPRINT
  //STEPLIB  DD DISP=SHR,DSN=TCNAMS1.LOAD
  //SYSLIN   DD DSN=*.ASMH.SYSLIN,DISP=(OLD,PASS)
  //SYSPRINT DD SYSOUT=*
  //SYSPUNCH DD DISP=(,CATLG),DSN=TCNAMS1.MULT,
  // UNIT=(3380,2),SPACE=(TRK,(1,1)),
  // RECFM=FB,LRECL=80,BLKSIZE=160
